@@ -2,11 +2,11 @@
 // anyRemote
 // a wi-fi or bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2014 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2016 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 //#define _GNU_SOURCE
@@ -47,6 +47,7 @@
 extern char         callerId[MAXLEN];
 extern type_key*    repeatCmdPtr;
 extern char         tmp[MAXMAXLEN];
+extern SingleList*  _peers;
 
 static int processCommands(SingleList* commands, cmdParams* p);
 
@@ -209,7 +210,7 @@ static string_t* substParams(const char* in, cmdParams *params)
             free(varName);
         }
     }
-    DEBUG2("[EX]: substParams: final cmd >%s<", cmdBuffer->str);
+    //DEBUG2("[EX]: substParams: final cmd >%s<", cmdBuffer->str);
 
     free(command);
 
@@ -218,8 +219,12 @@ static string_t* substParams(const char* in, cmdParams *params)
 
 static char* executeSimpleCommand(const char* exec, size_t* sz)
 {
-    if (exec == NULL || strlen(exec) == 0 || remoteOn != 1) {
-        logger(L_DBG, "[EX]: executeSimpleCommand null input or remote if OFF");
+    if (exec == NULL || strlen(exec) == 0) {
+        logger(L_DBG, "[EX]: executeSimpleCommand null input");
+        return NULL;
+    }
+    if (remoteOn != 1) {
+        logger(L_DBG, "[EX]: executeSimpleCommand remote if OFF");
         return NULL;
     }
 
@@ -252,21 +257,22 @@ static int processSendCmd(const char* sDescr)
     }
     char *ptr   = tag + strlen(tag) - 1;
     // strip spaces from tail
-    while( *ptr == '\0' || *ptr == '\n' || *ptr == '\t' || *ptr == ' ') {
+    while ( *ptr == '\0' || *ptr == '\n' || *ptr == '\t' || *ptr == ' ') {
         *ptr = '\0';
         ptr--;
     }
     ptr = tag + strlen(tag) + 1;
     // strip spaces from head of value
-    while( *ptr != '\0' && (*ptr == '\t' || *ptr == ' ')) {
+    while ( *ptr != '\0' && (*ptr == '\t' || *ptr == ' ')) {
         ptr++;
     }
 
     DEBUG2("[EX]: processSendCmd() is >%s< >%s<",tag,ptr);
 
-    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-    dm->size  = 1;
-    dm->file  = NULL;
+    dMessage* dm = allocDMessage();
+    dm->size     = 1;
+    dm->subtype  = ID_SET_MAX;
+    
     if (strcmp(tag, SEND_STRING) == 0) {
 
         dm->type  = DM_SENDS;
@@ -292,9 +298,9 @@ static void sendData(int type, int flag, char *data, int count)
 {
     DEBUG2("[EX]: sendData() >%d<",count);
 
-    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-    dm->size  = count;
-    dm->file  = NULL;
+    dMessage* dm = allocDMessage();
+    dm->size     = count;
+    dm->subtype  = ID_SET_MAX;
 
     if (type == ID_SEND && flag) {
         logger(L_DBG,"[EX]: sendData() send as bytes");
@@ -413,13 +419,6 @@ int handleHook(int hook)
     //sprintf(tmp, "uploadPix file >%s<", fname);
     //logger(L_INF,tmp);
 
-    if (strstr(cmdLine,UPLOAD_COVER_DATA) != NULL &&
-            isDataNew(ID_COVER, fname, strlen(fname)) != RC_OK) {
-
-        logger(L_INF,"Skip upload same cover");
-        return;
-    }
-
     FILE *fp;
     struct stat buf;
 
@@ -431,8 +430,9 @@ int handleHook(int hook)
         } else {
             logger(L_DBG,"uploadPix(): send empty cover");
 
-            dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-            dm->type  = DM_SET;
+            dMessage* dm = allocDMessage();
+            dm->type     = DM_SET;
+            dm->subtype  = ID_SET_MAX;
             dm->size  = 11;
             dm->value = (void*) strdup("Set(cover);");
 
@@ -481,8 +481,9 @@ int handleHook(int hook)
     }
     strcpy(fBuffer+prefixSz+fLen+8,");");
 
-    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-    dm->type  = DM_SET;
+    dMessage* dm = allocDMessage();
+    dm->type     = DM_SET;
+    dm->subtype  = ID_SET_MAX;
     dm->size  = prefixSz+fLen+10;
     dm->value = (void*) fBuffer;    // transfer ovnership to dispatcher
 
@@ -491,7 +492,8 @@ int handleHook(int hook)
     return;
 }*/
 
-void uploadPix(char *cmdLine)
+
+static void uploadPix(int subtype, char *cmdLine)
 {
     DEBUG2("[EX]: uploadPix >%s<", cmdLine);
     int prefixSz = 0;                  // image,icon,_name_,_file_|image,window,_file_|cover,noname, _file_
@@ -520,11 +522,6 @@ void uploadPix(char *cmdLine)
     sprintf(tmp, "[EX]: uploadPix file >%s<", fname);
     logger(L_INF,tmp);
 
-    if (strstr(cmdLine,UPLOAD_COVER_DATA) != NULL && isDataNew(ID_SET_COVER, fname, strlen(fname)) != RC_OK) {
-        logger(L_INF,"[EX]: Skip upload same cover");
-        return;
-    }
-
     char * fBuffer = (char*) calloc(prefixSz+6,1); // "Set(" + "," + "\0"
     if (fBuffer == NULL) {
         logger(L_ERR,"[EX]: no more memory!");
@@ -540,17 +537,262 @@ void uploadPix(char *cmdLine)
     }
     strcpy(fName,fname);
 
-    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-    dm->type  = DM_SETFILE;
-    dm->size  = -1;
-    dm->value = (void*) fBuffer;    // transfer ovnership to dispatcher
-    dm->file  = (void*) fName;
+    dMessage* dm = allocDMessage();
+    dm->peer     = PEER_ANY;    // TODO - use specific peer
+    dm->type     = DM_SETFILE;
+    dm->subtype  = subtype;
+    dm->size     = -1;
+    dm->value    = (void*) fBuffer;    // transfer ovnership to dispatcher
+    dm->file     = (void*) fName;
+    dm->scaled   = (void*) fName;
 
     sendToDispatcher(dm);
 
     return;
 }
 
+static void uploadCoverToPeers(const char* command, char* file, boolean_t isExec)
+{
+    if (!file) {
+        return;
+    }
+    int sz = strlen(file);
+    if (sz > 1 && *(file+sz-1) == '\n') {
+        *(file+sz-1) = '\0';
+    }
+    
+    DEBUG2("[EX]: uploadCoverToPeers %s<>%s", command, file);
+    
+    SingleList* list = _peers;
+    
+    char * fBuffer = (char*) calloc(strlen(command)+5,1);    // 4 = "Set(" + "\0"
+    if (fBuffer == NULL) {
+        logger(L_ERR,"no more memory!");
+        return;
+    }
+
+    strcpy(fBuffer, "Set(");
+    strcat(fBuffer, command);
+    if (isExec) {
+        strcat(fBuffer, ",");
+    }
+   
+    while (list) {
+    
+        PeerDef* peer = (PeerDef*) list->data;
+        DEBUG2("[EX]: Prepare cover for peer %d", peer->id);
+        
+        char buf[16];
+        sprintf(buf,"%d",peer->coverSz);
+         
+        char* dup = strdup(file);
+        int ssz = strlen(dup);
+        int c = ssz;
+        while (*(dup+c) != '/' && c >= 0) {
+            c--;
+        }
+        if (c != ssz) {
+            c++;
+        }
+        
+        int dot = ssz;
+        while (*(dup+dot) != '.' && dot >= 0) {
+           dot--;
+        }
+        if (dot > 0) {
+            *(dup+dot) = '\0';   
+        } 
+        DEBUG2("[EX]: Parsed file name %s", (dup+c));
+        
+        string_t* scaledFile = stringNew("/tmp/");
+        
+        stringAppend(scaledFile,dup+c);
+        stringAppend(scaledFile, "_");
+        stringAppend(scaledFile, buf);
+        stringAppend(scaledFile, ".png");
+        free(dup);
+        
+        DEBUG2("[EX]: Scaled file name %s", scaledFile->str);
+        
+        string_t* cmd = stringNew("convert -resize ");
+
+        stringAppend(cmd, buf);
+        stringAppend(cmd, "x");
+        stringAppend(cmd, buf);
+        stringAppend(cmd, " -depth 8 -background transparent ");
+        stringAppend(cmd, file);
+        stringAppend(cmd, " ");
+        stringAppend(cmd, scaledFile->str);
+        stringAppend(cmd, " 2> /dev/null");
+        
+        DEBUG2("[EX]: Scaled command %s", cmd->str);
+        
+        size_t sz = 0;
+        char* dummy = executeCommandPipe(cmd->str, &sz);
+        if (dummy) {  // can be NULL
+            free(dummy);
+        }
+        stringFree(cmd, BOOL_YES);
+        
+        DEBUG2("[EX]: uploadCoverToPeers Final command %s %s", fBuffer, scaledFile->str);
+
+        dMessage* dm = allocDMessage();
+        dm->peer     = peer->id;
+        dm->type     = DM_SETFILE;
+        dm->subtype  = ID_SET_COVER;
+        dm->size     = -1;
+        dm->value    = (void*) strdup(fBuffer);        // transfer ovnership to dispatcher
+        dm->file     = (void*) strdup(file);
+        dm->scaled   = (void*) strdup(scaledFile->str);
+        
+        stringFree(scaledFile, BOOL_YES);
+
+        sendToDispatcher(dm);
+
+        list = listSingleNext(list);
+    }
+    
+    free(fBuffer);
+}
+
+static void uploadCoverCmd(const char* descr, boolean_t isExec)
+{ 
+    char* dup = strdup(descr);
+    char* ptr = strstr(dup,",cover,");
+    ptr += 7;
+    while (*ptr != ',') {
+        ptr++;
+    }
+    ptr++;
+    char* f = strdup(ptr);
+    *ptr = '\0';
+    
+    uploadCoverToPeers(dup, f, isExec);
+    
+    free(dup);
+    free(f);
+}
+
+static void uploadWindowToPeers(const char* command, char* file)
+{
+    if (!file) {
+        return;
+    }
+    int sz = strlen(file);
+    if (sz > 1 && *(file+sz-1) == '\n') {
+        *(file+sz-1) = '\0';
+    }
+    
+    DEBUG2("[EX]: uploadWindowToPeers %s<>%s", command, file);
+    
+    SingleList* list = _peers;
+    
+    char * fBuffer = (char*) calloc(strlen(command)+5,1);    // 4 = "Set(" + "\0"
+    if (fBuffer == NULL) {
+        logger(L_ERR,"no more memory!");
+        return;
+    }
+
+    strcpy(fBuffer, "Set(");
+    strcat(fBuffer, command);
+   
+    while (list) {
+    
+        PeerDef* peer = (PeerDef*) list->data;
+        DEBUG2("[EX]: Prepare image for peer %d", peer->id);
+        
+        char bufx[16];
+        char bufy[16];
+        sprintf(bufx,"%d",peer->xSz);
+        sprintf(bufy,"%d",peer->ySz);
+         
+        char* dup = strdup(file);
+        int ssz = strlen(dup);
+        int c = ssz;
+        while (*(dup+c) != '/' && c >= 0) {
+            c--;
+        }
+        if (c != ssz) {
+            c++;
+        }
+        
+        int dot = ssz;
+        while (*(dup+dot) != '.' && dot >= 0) {
+           dot--;
+        }
+        if (dot > 0) {
+            *(dup+dot) = '\0';   
+        } 
+        DEBUG2("[EX]: Parsed file name %s", (dup+c));
+        
+        string_t* scaledFile = stringNew("/tmp/");
+        
+        stringAppend(scaledFile,dup+c);
+        stringAppend(scaledFile, "_");
+        stringAppend(scaledFile, bufx);
+        stringAppend(scaledFile, "x");
+        stringAppend(scaledFile, bufy);
+        stringAppend(scaledFile, ".png");
+        free(dup);
+        
+        DEBUG2("[EX]: Scaled file name %s", scaledFile->str);
+        
+        string_t* cmd = stringNew("convert -resize ");
+
+        stringAppend(cmd, bufx);
+        stringAppend(cmd, "x");
+        stringAppend(cmd, bufy);
+        stringAppend(cmd, " -depth 8 -background transparent ");
+        stringAppend(cmd, file);
+        stringAppend(cmd, " ");
+        stringAppend(cmd, scaledFile->str);
+        stringAppend(cmd, " 2> /dev/null");
+        
+        DEBUG2("[EX]: Scaled command %s", cmd->str);
+        
+        size_t sz = 0;
+        char* dummy = executeCommandPipe(cmd->str, &sz);
+        if (dummy) {  // can be NULL
+            free(dummy);
+        }
+        stringFree(cmd, BOOL_YES);
+        
+        DEBUG2("[EX]: uploadWindowToPeers Final command %d %s %s", peer->id, fBuffer, scaledFile->str);
+
+        dMessage* dm = allocDMessage();
+        dm->peer     = peer->id;
+        dm->type     = DM_SETFILE;
+        dm->subtype  = ID_SET_IMAGE;
+        dm->size     = -1;
+        dm->value    = (void*) strdup(fBuffer);        // transfer ovnership to dispatcher
+        dm->file     = (void*) strdup(file);
+        dm->scaled   = (void*) strdup(scaledFile->str);
+        
+        stringFree(scaledFile, BOOL_YES);
+
+        sendToDispatcher(dm);
+
+        list = listSingleNext(list);
+    }
+    
+    free(fBuffer);
+}
+
+static void uploadWindowCmd(const char* descr)
+{ 
+    DEBUG2("[EX]: uploadWindowCmd %s",descr);
+    char* dup = strdup(descr);
+    char* ptr = strstr(dup,",window,");
+    ptr += 8;
+    char* f = strdup(ptr);
+    *ptr = '\0';
+    
+    uploadWindowToPeers(dup, f);
+    
+    free(dup);
+    free(f);
+}
+
 static int isConvertable(int what)
 {
     if (what == ID_SET_LIST || what == ID_SET_ILIST || what == ID_SET_FMGR) {
@@ -598,13 +840,13 @@ static int getCmd(int subtype, const char *descr, const char *exec, cmdParams* p
     strcat(tmptext, descr);
     strcat(tmptext, ");");
 
-    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
+    dMessage* dm = allocDMessage();
     dm->size  = strlen(tmptext);
     dm->value = strdup(tmptext);
-    dm->file  = NULL;
 
     if (isServerMode()) {
-        dm->type  = DM_SET;
+        dm->type    = DM_GET;
+        dm->subtype = subtype;
     } else {
         free(dm->value);
         free(dm);
@@ -621,7 +863,7 @@ static int getCmd(int subtype, const char *descr, const char *exec, cmdParams* p
 
 static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdParams* params)
 {
-    logger(L_INF, "[EX]: Command: ExecAndSet");
+    INFO2("[EX]: Command: ExecAndSet %d", subtype);
 
     if (!isServerMode() || remoteOn != 1) {
         return EXIT_OK;
@@ -644,24 +886,36 @@ static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdPa
 
     int dlen = strlen(descr);
 
-    if (stringHasPrefix(descr,UPLOAD_ICON)  || 
-        stringHasPrefix(descr,UPLOAD_PIX)   ||
-        stringHasPrefix(descr,UPLOAD_COVER) ||
-        (subtype == ID_SET_COVER && strstr(descr,",noname,") != NULL)) {
+    if (subtype == ID_SET_IMAGE) {
         
         INFO2("[EX]: ExecAndSet upload icon/image/cover of size %d",(int)resLen);
-       
+
         char *chunk = (char*) calloc(dlen + resLen + 2, 1);
 
         strcpy(chunk,descr);
         strcat(chunk,",");
         memcpy(chunk + dlen + 1, res, resLen);
-
-        uploadPix(chunk);
-
+        
+        DEBUG2("[EX]: ExecAndSet upload icon/image/cover %s", descr);
+        
+        if (strstr(descr,",cover") != NULL) {
+            uploadCoverCmd(chunk, BOOL_NO);
+        } else if (strstr(descr,",window") != NULL) {
+            uploadWindowCmd(chunk);
+        } else {
+            if (strstr(descr,",icon") != NULL) {
+                uploadPix(subtype, chunk);
+            }
+            // Set(image,show|close|cursor|nocursor|remove_all) processed below
+        }
         free(chunk);
+
+    } else if (subtype == ID_SET_COVER && strstr(descr,",noname") != NULL) {
+        
+        logger(L_INF, "[EX]: ExecAndSet upload cover");
+        uploadCoverToPeers(descr, res, BOOL_YES);
    
-    } else if (subtype == ID_SET_COVER && strstr(descr,",by_name,") != NULL) {
+    } else if (subtype == ID_SET_COVER && strstr(descr,",by_name") != NULL) {
          
         string_t* command = stringNew(CMD_SET);
         stringAppend(command,"(");
@@ -670,11 +924,11 @@ static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdPa
         stringAppendLen(command,res,resLen);
         stringAppend(command,");");
 
-        dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-        dm->type  = DM_SET;
-        dm->size  = command->len;
-        dm->value = (void*) strdup(command->str);
-        dm->file  = NULL;
+        dMessage* dm = allocDMessage();
+        dm->type     = DM_SET;
+        dm->subtype  = ID_SET_COVER;
+        dm->size     = command->len;
+        dm->value    = (void*) strdup(command->str);
         
         stringFree(command,BOOL_YES);
 
@@ -682,74 +936,68 @@ static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdPa
 
     } else {
 
-        if (isDataNew(subtype, res, resLen) != RC_OK) {
-            logger(L_INF, "[EX]: Skip to send the same data");
-        } else {
-            logger(L_INF, "[EX]: Data is new");
-
-            // Replace all ");" inside buffer to "@@" since Java Client will separate commands by ");"
-            char *p;
-            while ((p = strstr(res,");")) != NULL) {        // binary data ???
-                *p = '@';
-                p++;
-                *p = '@';
-            }
-
-            boolean_t converted = BOOL_NO;
-
-            #ifdef USE_ICONV
-            char* decodedVal = res;
-            if (isConvertable(subtype) && needConvert() == 1) {
-                decodedVal = convCharset(res, resLen, CNV_TO);
-                converted = BOOL_YES;
-                DEBUG2("[EX]: Decoded string >%s< ", (decodedVal ? decodedVal : "NULL"));
-            }
-            int sz = (decodedVal ? strlen(decodedVal) : 0);
-
-            int memSize = sz + dlen + 7;
-
-            #else
-
-            int memSize = resLen + dlen + 7;
-
-           #endif
-
-            char *chunk = (char*) calloc(1,memSize+1);
-            strcpy(chunk,CMD_SET);
-
-            strcat(chunk,"(");  // For example, Set(title
-            strcat(chunk,descr);
-
-            strcat(chunk,",");
-
-            #ifdef USE_ICONV
-
-            memcpy(chunk+5+dlen,decodedVal,sz);      // could be binary data here ????
-            strcpy(chunk+5+dlen+sz,");");
-            
-            if (converted == BOOL_YES) {
-                free(decodedVal);
-            }
-
-            #else
-
-            memcpy(chunk+5+dlen,res,resLen); // could be binary data here
-            strcpy(chunk+5+dlen+resLen,");");
-
-            #endif
-
-            //printf("CHUNK: %s\n", chunk);
-
-            dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-            dm->type  = DM_SET;
-            dm->size  = strlen(chunk);
-            dm->value = (void*) strdup(chunk);
-            dm->file  = NULL;
-
-            sendToDispatcher(dm);
-
-            free(chunk);
+        // Replace all ");" inside buffer to "@@" since Java Client will separate commands by ");"
+        char *p;
+        while ((p = strstr(res,");")) != NULL) {        // binary data ???
+            *p = '@';
+            p++;
+            *p = '@';
         }
+
+        boolean_t converted = BOOL_NO;
+
+        #ifdef USE_ICONV
+        char* decodedVal = res;
+        if (isConvertable(subtype) && needConvert() == 1) {
+            decodedVal = convCharset(res, resLen, CNV_TO);
+            converted = BOOL_YES;
+            DEBUG2("[EX]: Decoded string >%s< ", (decodedVal ? decodedVal : "NULL"));
+        }
+        int sz = (decodedVal ? strlen(decodedVal) : 0);
+
+        int memSize = sz + dlen + 7;
+
+        #else
+
+        int memSize = resLen + dlen + 7;
+
+        #endif
+
+        char *chunk = (char*) calloc(1,memSize+1);
+        strcpy(chunk,CMD_SET);
+
+        strcat(chunk,"(");  // For example, Set(title
+        strcat(chunk,descr);
+
+        strcat(chunk,",");
+
+        #ifdef USE_ICONV
+
+        memcpy(chunk+5+dlen,decodedVal,sz);      // could be binary data here ????
+        strcpy(chunk+5+dlen+sz,");");
+        
+        if (converted == BOOL_YES) {
+            free(decodedVal);
+        }
+
+        #else
+
+        memcpy(chunk+5+dlen,res,resLen); // could be binary data here
+        strcpy(chunk+5+dlen+resLen,");");
+
+        #endif
+
+        //printf("CHUNK: %s\n", chunk);
+
+        dMessage* dm = allocDMessage();
+        dm->type     = DM_SET;
+        dm->subtype  = subtype;
+        dm->size     = strlen(chunk);
+        dm->value    = (void*) strdup(chunk);
+
+        sendToDispatcher(dm);
+
+        free(chunk);
     }
 
     free(res);
@@ -804,19 +1052,27 @@ static int setCmd(int subtype, const char *descr, const char *exec, cmdParams* p
 {
     INFO2("[EX]: Command: Set %s", (descr ? descr : "NULL"));
     if (descr != NULL && isServerMode() == EXIT_OK) {
-       
 
-        if (strstr(descr,UPLOAD_ICON)  == descr ||        // handle Set(cover|image,icon|window|cover,[...]_image_file_name_); separately.
-            strstr(descr,UPLOAD_PIX)   == descr ||
-            strstr(descr,UPLOAD_COVER) == descr) {
+        if (subtype == ID_SET_IMAGE) {
+            // handle Set(cover|image,icon|window|cover,[...]_image_file_name_); separately.
 
-            uploadPix((char*)descr);
+            if (strstr(descr,",cover,") != NULL) {
+                uploadCoverCmd(descr, BOOL_NO);
+            } else if (strstr(descr,",window,") != NULL) {
+                uploadWindowCmd(descr);
+            } else {
+                if (strstr(descr,",icon,") != NULL) {
+                    uploadPix(subtype, (char*)descr);
+                } 
+                // Set(image,show|close|cursor|nocursor|remove_all) processed below
+            }
             
         } else if (subtype == ID_SET_COVER) {
         
             if (strstr(descr,",noname,") != NULL) {
             
-                uploadPix((char*)descr);
+                char* ptr = strstr(descr,",noname,");
+                uploadCoverToPeers("cover,noname", (ptr+8), BOOL_NO);
                 
             } else if (strstr(descr,",by_name,") != NULL ||
                        strstr(descr,",clear") != NULL) {
@@ -826,57 +1082,133 @@ static int setCmd(int subtype, const char *descr, const char *exec, cmdParams* p
                 stringAppend(command,descr);
                 stringAppend(command,");");
 
-                dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-                dm->type  = DM_SET;
-                dm->size  = command->len;
-                dm->value = (void*) strdup(command->str);
-                dm->file  = NULL;
+                dMessage* dm = allocDMessage();
+                dm->type     = DM_SET;
+                dm->subtype  = subtype;
+                dm->size     = command->len;
+                dm->value    = (void*) strdup(command->str);
 
                 stringFree(command,BOOL_YES);
                 
                 sendToDispatcher(dm);
             }
+        
+        } else if ((subtype == ID_SET_LIST && strncmp(descr,"list,dir,",9) == 0) || 
+                   (subtype == ID_SET_ILIST && strncmp(descr,"iconlist,dir,",13) == 0)) {
+           
+           INFO2("[EX]: Command: Set list from dir %s", (descr ? descr : "NULL"));
+           string_t* command = stringNew(CMD_SET);
+           
+           int shift;
+           if (subtype == ID_SET_LIST) {
+               stringAppend(command,"(list,replace,");
+               shift = 9;
+           } else {
+               stringAppend(command,"(iconlist,replace,");
+               shift = 13;
+           }
+           const char *p1 = descr + shift;
+           const char *p2 = p1;
+           while (*p2 != ',') {
+               p2++;
+           }
+           p2++;
+           stringAppendLen(command, p1, p2-p1);
+           
+           
+           INFO2("[EX]: Command: Set list from dir2 %s", p2);
+           
+           string_t* ff = executeDirListCommand(subtype, p2);
+           
+           if (ff) {
+               stringAppend(command, ff->str);
+               stringFree(ff,BOOL_YES);
+           }
+           
+           stringAppend(command,");");
+
+           dMessage* dm = allocDMessage();
+           dm->type     = DM_SET;
+           dm->subtype  = subtype;
+           dm->size     = command->len;
+           dm->value    = (void*) strdup(command->str);
+
+           stringFree(command,BOOL_YES);
+           
+           sendToDispatcher(dm);
+
+        } else if (subtype == ID_SET_TEXT && strncmp(descr,"text,file,",10) == 0) {
+           
+           INFO2("[EX]: Command: Set text from file %s", (descr ? descr : "NULL"));
+           string_t* command = stringNew(CMD_SET);
+           
+           stringAppend(command,"(text,replace,");
+
+           const char *p1 = descr + 10;  // text,file
+           const char *p2 = p1;
+           while (*p2 != ',') {
+               p2++;
+           }
+           p2++;
+           stringAppendLen(command, p1, p2-p1);
+           
+           
+           INFO2("[EX]: Command: Set text from file %s", p2);
+           int len;
+           char *buf = readFromFile("", p2, &len);
+           if (buf) {
+               stringAppendLen(command, buf, len);
+               free(buf);
+           }
+           
+           stringAppend(command,");");
+
+           dMessage* dm = allocDMessage();
+           dm->type     = DM_SET;
+           dm->subtype  = subtype;
+           dm->size     = command->len;
+           dm->value    = (void*) strdup(command->str);
+
+           stringFree(command,BOOL_YES);
+           
+           sendToDispatcher(dm);
 
         } else {
 
+            //INFO2("[EX]: Command: Set ELSE %s", (descr ? descr : "NULL"));
+            
             const char* data = descr;
-
-            if (isDataNew(subtype, data, strlen(data)) != RC_OK) {
-                logger(L_INF, "[EX]: Skip to send the same data");
-            } else {
-                logger(L_INF, "[EX]: Data is new");
         
-                string_t* command = stringNew(CMD_SET);
-                stringAppend(command,"(");
+            string_t* command = stringNew(CMD_SET);
+            stringAppend(command,"(");
 
-                // Change \n in input to real '\n'
-                const char *p1 = data;
-                const char *p2 = data;
+            // Change \n in input to real '\n'
+            const char *p1 = data;
+            const char *p2 = data;
 
-                while (p2 != NULL) {
-                    p2 = strstr(p1, "\\n");
+            while (p2 != NULL) {
+                p2 = strstr(p1, "\\n");
 
-                    if (p2 == NULL) {
-                         stringAppend(command,p1);
-                    } else {
-                         stringAppendLen(command,p1, p2-p1);
-                         stringAppend(command,"\n");
-                        p1 = p2 + 2;
-                    }
+                if (p2 == NULL) {
+                     stringAppend(command,p1);
+                } else {
+                     stringAppendLen(command,p1, p2-p1);
+                     stringAppend(command,"\n");
+                    p1 = p2 + 2;
                 }
-
-                 stringAppend(command,");");
-
-                dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-                dm->type  = DM_SET;
-                dm->size  = command->len;
-                dm->value = (void*) strdup(command->str);
-                dm->file  = NULL;
-
-                sendToDispatcher(dm);
-        
-                stringFree(command,BOOL_YES);
             }
+
+            stringAppend(command,");");
+
+            dMessage* dm = allocDMessage();
+            dm->type     = DM_SET;
+            dm->subtype  = subtype;
+            dm->size     = command->len;
+            dm->value    = (void*) strdup(command->str);
+
+            sendToDispatcher(dm);
+        
+            stringFree(command,BOOL_YES);
         }
     }
 
@@ -938,12 +1270,12 @@ static int makeModeCmd(const char *descr, const char *exec)
     type_key* k = findItem(EVT_EXITMODE, &i, NULL);
     if (k && i == FLAG_EXACT) {
         logger(L_DBG, "[EX]: Exec cmd on exit from mode");
-        ret = handleCmdByKey(k,NULL);
+        ret = handleCmdByKey(PEER_ANY,k,NULL);
         if (ret == EXIT_STOP) {
             return ret;
         }
-    } else {
-        logger(L_DBG, "[EX]: Leave mode");
+    //} else {
+    //    logger(L_DBG, "[EX]: Leave mode");
     }
 
     switchMode(exec);
@@ -952,7 +1284,7 @@ static int makeModeCmd(const char *descr, const char *exec)
     k = findItem(EVT_ENTRMODE, &i, NULL);
     if (k && i == FLAG_EXACT) {
         logger(L_DBG, "[EX]: Exec cmd on enter to mode");
-        ret = handleCmdByKey(k,NULL);
+        ret = handleCmdByKey(PEER_ANY,k,NULL);
         if (ret == EXIT_STOP) {
             return ret;
         }
@@ -999,13 +1331,9 @@ static int makeDisconnectCmd(const char *descr, const char *exec)
 {
     logger(L_INF, "[EX]: Temporary disconnect command");
 
-    freeCachedData();
-
-    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-    dm->size  = 0;
-    dm->value = (void*) NULL;
-    dm->type  = DM_EVENT_DISCONNECT;
-    dm->file  = NULL;
+    dMessage* dm = allocDMessage();
+    dm->type     = DM_EVENT;
+    dm->subtype  = ID_EVENT_DISCONNECT;
 
     sendToDispatcher(dm);
 
@@ -1067,11 +1395,11 @@ static int ckpdCmd(int subtype, const char *descr, const char *exec, cmdParams*
     }
 
     if (isAtModeDuplex()) {
-        dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-        dm->type  = DM_CKPD;
-        dm->size  = strlen(descr);
-        dm->value = (void*) strdup(descr);
-        dm->file  = NULL;
+        dMessage* dm = allocDMessage();
+        dm->type     = DM_CKPD;
+        dm->subtype  = ID_SET_MAX;
+        dm->size     = strlen(descr);
+        dm->value    = (void*) strdup(descr);
 
         sendToDispatcher(dm);
     }
@@ -1108,7 +1436,7 @@ int macroCmd(int subtype, const char *descr, const char *exec, cmdParams* params
                 res = executeSimpleCommand(eValue, &resLen);
         
                 if (res == NULL) {
-            logger(L_ERR, "[EX]: macroCmd: empty condition value, skip execution");
+                    logger(L_ERR, "[EX]: macroCmd: empty condition value, skip execution");
                     return EXIT_NOK;
                 }
                 if (*(res+resLen-1) == '\n') {
@@ -1132,7 +1460,7 @@ int macroCmd(int subtype, const char *descr, const char *exec, cmdParams* params
                 runIt = 0;
             }
         }
-    free(res);
+        free(res);
     }
 
     if (runIt == 0) {
@@ -1144,7 +1472,7 @@ int macroCmd(int subtype, const char *descr, const char *exec, cmdParams* params
     type_key *k = findItem(descr, &found, NULL);
 
     if (k && found == FLAG_EXACT) {
-        ret = handleCmdByKey(k, params);
+        ret = handleCmdByKey(PEER_ANY,k, params);
         DEBUG2("[EX]: Return code from macroCmd -> handleCmdByKey() is %d",ret);
     }
 
@@ -1160,8 +1488,7 @@ int macroCmdCallback(const char *descr, const char *exec, cmdParams* params)
     string_t* d = substParams(descr, params);
     string_t* e = substParams(exec,  params);
     
-    INFO2("[EX]: macroCmdCallback after subst >%s< -> >%s<",
-          (d ? d->str : "NULL"), (e ? e->str : "NULL"));
+    INFO2("[EX]: macroCmdCallback after subst >%s< -> >%s<", (d ? d->str : "NULL"), (e ? e->str : "NULL"));
 
     int ret = macroCmd(-1, (d ? d->str : NULL), (e ? e->str : NULL), params);
 
@@ -1206,8 +1533,9 @@ static int processOneCommand(cmdItem* ci, cmdParams* p)
           ci->descr == NULL ? "no description" : ci->descr,
           ci->exec  == NULL ? "no exec"        : ci->exec, remoteOn);
 
-    if (ci->type == ID_MAKE && ci->subtype == ID_MAKE_REMOTE &&
-            ci->exec && strcmp(ci->exec,"on") == 0) {
+    if (ci->type == ID_MAKE && 
+        ci->subtype == ID_MAKE_REMOTE &&
+        ci->exec && strcmp(ci->exec,"on") == 0) {
 
         logger(L_INF, "[EX]: Switch remote on");
         remoteOn = 1;
@@ -1253,12 +1581,13 @@ static int processCommands(SingleList* commands, cmdParams* p)
     cmdItem * item = (cmdItem *) list->data;
 
         /*if (item->descr == NULL) {
-                    sprintf(tmp, "Process next command NULL parameters");
-            } else {
-                    sprintf(tmp, "Process next command %s",item->descr);
-            }
-            logger(L_DBG, tmp);
-            */
+            sprintf(tmp, "Process next command NULL parameters");
+        } else {
+            sprintf(tmp, "Process next command %s",item->descr);
+        }
+        logger(L_DBG, tmp);
+        */
+        
         int ret = processOneCommand(item, p);
     
         if (ret != EXIT_OK) {
@@ -1277,12 +1606,12 @@ static int processCommands(SingleList* commands, cmdParams* p)
 //
 // Top-level command processing function
 //
-int handleCmdByKey(type_key* k, cmdParams* p)
+int handleCmdByKey(int peer, type_key* k, cmdParams* p)
 {
-    return handleCmdByKeyEx(k, p, 1);
+    return handleCmdByKeyEx(peer, k, p, 1);
 }
 
-int handleCmdByKeyEx(type_key* k, cmdParams* p, int sendToMain)
+int handleCmdByKeyEx(int peer, type_key* k, cmdParams* p, int sendToMain)
 {
     if (k == NULL) {
         logger(L_DBG, "[EX]: handleCmdByKey() NULL input");
@@ -1305,8 +1634,8 @@ int handleCmdByKeyEx(type_key* k, cmdParams* p, int sendToMain)
         // In AT mode: User had pressed some key ...
         // Send ToMainMenu sequence of CKPD's to show main menu again
         // Do we send it even if remote is off ?
-        if (remoteOn == 1 && isAtModeDuplex() && sendToMain == 1) {
-            sendToMainMenu();
+        if (remoteOn == 1 && peer != PEER_ANY && needAtMainMenuReturn(peer) && sendToMain == 1) {
+            sendToMainMenu(peer);
         }
     }
 
