@@ -2,11 +2,11 @@
 // anyRemote
 // a wi-fi or bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2014 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2016 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #ifndef _COMMON_H_
@@ -38,11 +38,11 @@
 #define IS_OK(answer) (strstr(answer, "OK") != NULL)
 
 //#define DEFAULT_DEVICE  "/dev/rfcomm0"	seems JSR-82 phones is more usable now
-//#define DEFAULT_DEVICE  "bluetooth:19"
+//#define DEFAULT_DEVICE  "bluetooth"
 
 #ifdef USE_AVAHI
   #if defined USE_BLUEZ || defined USE_BT_FBSD 
-    #define DEFAULT_DEVICE  "bluetooth:19,tcp:5197,web:5080,avahi"
+    #define DEFAULT_DEVICE  "bluetooth,tcp:5197,web:5080,avahi"
   #else
     #define DEFAULT_DEVICE  "tcp:5197,web:5080,avahi"
   #endif
@@ -68,16 +68,13 @@
 #define CMXML_SOCKET   	"cmxml:"
 #define AVAHI_USE   	"avahi"
 
-#define TOFILE   	"/anyremote.result"
-#define CMDFILE   	"/anyremote.command"
-
-#define KEEPALIVE_TIMER "_PING_TIMER_"
-
 #define PROTO_AT        1
 #define PROTO_ANYREMOTE 2
 #define PROTO_BEMUSED   3
 #define PROTO_IVIEWER   4
 
+#define PEER_ANY        0
+
 enum ProtocolMode {
     CLIENT_RFCOMM = 0,	 // AT protocol over bluetooth
     CLIENT_AT,	         // AT protocol over IR or cable
@@ -88,12 +85,15 @@ enum ProtocolMode {
     SERVER_STDIN,	     // anyRemote protocol, read command from stdin
     SERVER_WEB,	         // anyRemote protocol, built-in web server
     SERVER_CMXML,	     // anyRemote protocol, built-in XML server
+    #ifdef USE_L2CAP
     SERVER_L2CAP,	     // TODO: anyRemote protocol over l2cap
+    #endif
     SERVER_UX ,  	     // NOT USED: anyRemote protocol over IR (or any AF_UNIX socket) - as server
+    FRONT_END ,  	     // Used to communicate with frontend
     SERVER_MAX   	 
 };
 
-#define DEFAULT_BT_CHANNEL 19
+#define DEFAULT_BT_CHANNEL 1
 #define BT_ADDR_LEN        17   // 00:12:EF:32:21:1A
 #define DEFAULT_L2CAP_PORT 5019
 
@@ -103,6 +103,7 @@ enum ProtocolMode {
 
 #define VAR_AUTOCONN   	  "AutoConnect"
 #define VAR_AUTOREPEAT	  "AutoRepeat"
+#define VAR_CFGDIR    	  "CfgDir"
 #define VAR_CHARSET    	  "CharSet"
 #define VAR_TO_ENCODING   "ToEncoding"
 #define VAR_FROM_ENCODING "FromEncoding"
@@ -112,7 +113,9 @@ enum ProtocolMode {
 #define VAR_WAITEXEC      "WaitSeconds"
 #define VAR_ALLOWED_ONLY  "AllowedOnly"
 
-#define CMD_STR_DISCONNECT "Set(disconnect);"
+#define CMD_STR_DISCONNECT  "Set(disconnect);"
+#define CMD_STR_GETCOVERSZ  "Get(cover_size);"
+#define CMD_STR_GETSCREENSZ "Get(screen_size);"
 
 // TODO: Support L2CAP under FreeBSD
 //#ifdef USE_BLUEZ
