@@ -2,11 +2,11 @@
 // anyRemote
 // a wi-fi or bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2014 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2016 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #include <ctype.h>
@@ -46,7 +46,7 @@ extern char	tmp[MAXMAXLEN];
 extern int  gotSignal;
 extern boolean_t stillRun;
 
-static int  handle_command(char* cmd);
+static int  handle_command(int peer, char* cmd);
 static int  handle_alarm  (char* cmd) ;
 
 static int  isBemusedCommand(char *cmd);
@@ -68,6 +68,8 @@ char modifierString [MAXARGLEN];
 
 int  flushConf    = 0;
 
+SingleList * _peers = NULL;
+
 void freeEMessage(void* ptr)
 {
     eMessage *em = (eMessage *) ptr;
@@ -82,18 +84,81 @@ void sendToExecutor(eMessage *buf)
     queuePush(Q_EXEC, (void*) buf);
 }
 
+static void freePeerDataList(void* data)
+{
+    PeerDef* peer = (PeerDef* ) data;
+    free(peer);
+}
+
+static void freePeerData(int id)
+{
+    SingleList* list = _peers;
+    
+    while (list) {
+        PeerDef* peer = (PeerDef*) list->data;
+        
+        if (peer->id == id) {
+            DEBUG2("[EX]: Free peer %d data", id);
+            list = listSingleRemove(_peers, peer);
+            freePeerDataList(peer);
+            return;
+        }
+        list = listSingleNext(list);
+    }
+}
+
+static void executorCleanup()
+{
+    listSingleFullFree(_peers, freePeerDataList);
+}
+
+void customizePeer(int id, int szX, int szY, int coverSz)
+{
+    
+    // MUTEX ???
+    SingleList* list = _peers;
+    
+    while (list) {
+        PeerDef* peer = (PeerDef*) list->data;
+        
+        if (peer->id == id) {
+            if (szX != -1) {
+	        peer->xSz = szX;
+	    }
+            if (szY != -1) {
+                peer->ySz     = szY;
+            }
+	    if (coverSz != -1) {
+                peer->coverSz = coverSz;
+            }
+	    return;
+        }
+        list = listSingleNext(list);
+    }
+
+    PeerDef* pd = (PeerDef*) malloc(sizeof(PeerDef));
+    pd->id  = id;
+    pd->xSz = szX; 
+    pd->ySz = szY; 
+    pd->coverSz = coverSz; 
+    DEBUG2("[  ]: Peer data: Sizes %d -> %d %d %d", id, pd->xSz, pd->ySz, pd->coverSz);
+    _peers = listSingleAppend(_peers, pd);
+}
+
 static eMessage* getEFinalizer()
 {
     eMessage* em = (eMessage*) malloc(sizeof(eMessage));
+    em->peer  = PEER_ANY;
     em->type  = EM_AS_IS;
     em->value = strdup("End();");
 
     return em;
 }
 
-void sendEventToExecutor(int event)
+void sendEventToExecutor(int peer, int event)
 {
     eMessage* em = (eMessage*) malloc(sizeof(eMessage));
+    em->peer  = peer;
     em->type  = EM_EVENT;
     em->value = malloc(sizeof(int));
     *((int *)em->value) = event;
@@ -116,11 +181,9 @@ static void initExecutor()
     setInitDone();
     printConf();
 
-    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-    dm->size  = 0;
-    dm->value = (void*) NULL;
-    dm->type  = DM_EVENT_INIT;
-    dm->file  = NULL;
+    dMessage* dm = allocDMessage();
+    dm->type     = DM_EVENT;
+    dm->subtype  = ID_EVENT_INIT;
 
     sendToDispatcher(dm);
 }
@@ -137,7 +200,7 @@ static void doAutoRepeat()
             return;
         } else if (repeatTimer > 0 && isOdd == repeatTimer) {
             DEBUG2("[EX]: Auto repeat command ... (%d)", repeatTimer);
-            handleCmdByKey(repeatNow(),NULL);
+            handleCmdByKey(PEER_ANY, repeatNow(),NULL);
         }
         repeatTimer++;
     }
@@ -156,16 +219,16 @@ static void doTimers()
     }
 }
 
-static int handleKeyMsg(void * ptr)
+static int handleKeyMsg(int peer, void * ptr)
 {
     char * cmd = (char *) ptr;
-    sprintf(tmp, "[EX]: got key >%s<", (cmd ? cmd : "NULL"));
+    sprintf(tmp, "[EX]: (%d) got key >%s<", peer, (cmd ? cmd : "NULL"));
     logger(L_DBG, tmp);
-    handle_command(cmd);
+    handle_command(peer, cmd);
     return 0;
 }
 
-static int handleStringMsg(void * ptr)
+static int handleStringMsg(int peer, void * ptr)
 {
     char * msgIn = (char*) ptr;
     sprintf(tmp, "[EX]: got string >%s<", msgIn);
@@ -174,7 +237,7 @@ static int handleStringMsg(void * ptr)
     return 0;
 }
 
-static int handleEventMsg(void * ptr)
+static int handleEventMsg(int peer, void * ptr)
 {
     int* evt = (int*) ptr;
 
@@ -185,7 +248,7 @@ static int handleEventMsg(void * ptr)
         handleConnect();
     } else if (*evt == ID_EVT_DISCONNECT) {
         logger(L_DBG, "[EX]: got event EVT_DISCONNECT");
-        handleDisconnect();
+        handleDisconnect(peer);
     //} else if (*evt == ID_EVT_INIT) {
         //logger(L_DBG, "[EX]: got event EVT_INIT");
         //  handleInit();
@@ -197,7 +260,7 @@ static int handleEventMsg(void * ptr)
     return exitFlag;
 }
 
-static int handleAlarmMsg(void * ptr)
+static int handleAlarmMsg(int peer, void * ptr)
 {
     char * cmd = (char *) ptr;
     sprintf(tmp, "[EX]: got alarm >%s<", cmd);
@@ -206,16 +269,16 @@ static int handleAlarmMsg(void * ptr)
     return 0;
 }
 
-static int handleAsIsMsg(void * ptr)
+static int handleAsIsMsg(int peer, void * ptr)
 {
     logger(L_DBG, "[EX]: got as is");
     char * cmd = (char *) ptr;
 
-    dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-    dm->type  = DM_SET;
-    dm->size  = strlen(cmd);
-    dm->value = (void*) strdup(cmd);
-    dm->file  = NULL;
+    dMessage* dm = allocDMessage();
+    dm->type     = DM_SET;
+    dm->subtype  = ID_SET_MAX;
+    dm->size     = strlen(cmd);
+    dm->value    = (void*) strdup(cmd);
 
     sendToDispatcher(dm);
     return 0;
@@ -223,7 +286,7 @@ static int handleAsIsMsg(void * ptr)
 
 static struct {
     int id;
-    int (*hook)(void* p);
+    int (*hook)(int peer, void* p);
 } _msgHooks[] = {
     {EM_KEY,    handleKeyMsg   },
     {EM_STRING, handleStringMsg},
@@ -246,9 +309,10 @@ pointer_t executorRoutine(pointer_t thread)
 
         eMessage *em = (eMessage *) queuePop(Q_EXEC);
         if (em != NULL) {
-            //logger(L_DBG, "[EX]: got event");
+            
+            DEBUG2("[EX]: got event from %d", em->peer);
 
-            int exitFlag = (_msgHooks[em->type].hook)(em->value);
+            int exitFlag = (_msgHooks[em->type].hook)(em->peer, em->value);
 
             freeEMessage(em);
 
@@ -272,7 +336,9 @@ pointer_t executorRoutine(pointer_t thread)
         //logger(L_DBG, "[EX]: wait a bit");
         usleep(20000); // loop timer (1/50 of second)
     }
-
+    
+    executorCleanup();
+    
     DEBUG2("[EX]: thread stopped");
     return NULL;
 }
@@ -283,7 +349,7 @@ static void handleInit()
     type_key *k = findItem(EVT_INIT, &i, NULL);
     if (k && i == FLAG_EXACT) {
         logger(L_INF, "[EX]: Exec cmd on init");
-        handleCmdByKey(k,NULL);
+        handleCmdByKey(PEER_ANY,k,NULL);
     }
     handleHook(ID_EVT_INIT);
 }
@@ -297,7 +363,7 @@ static void handleConnect()
         logger(L_INF, "[EX]: Exec cmd on connect");
         ok1 = EXIT_OK;
 
-        handleCmdByKey(k,NULL);
+        handleCmdByKey(PEER_ANY,k,NULL);
     }
     int ok2 = handleHook(ID_EVT_CONNECT);
 
@@ -308,20 +374,19 @@ static void handleConnect()
     }
 }
 
-static void handleDisconnect()
+static void handleDisconnect(int peer)
 {
     int i = 0;
 
     cleanAutoRepeatFlag();
+    freePeerData(peer);
 
     type_key *k = findItem(EVT_DISCONNECT, &i, NULL);
     if (k && i == FLAG_EXACT) {
         logger(L_INF, "[EX]: Exec cmd on disconnect");
-        handleCmdByKey(k,NULL);
+        handleCmdByKey(PEER_ANY,k,NULL);
     }
     handleHook(ID_EVT_DISCONNECT);
-
-    freeCachedData();
 }
 
 static void handleExit()
@@ -336,7 +401,7 @@ static void handleExit()
     type_key *k = findItem(EVT_EXIT, &i, NULL);
     if (k && i == FLAG_EXACT) {
         logger(L_INF, "[EX]: Exec cmd on exit");
-        handleCmdByKey(k,NULL);   // Since we exiting now we did not interested in return code
+        handleCmdByKey(PEER_ANY,k,NULL);   // Since we exiting now we did not interested in return code
     }
 
     //printf("[EX]: handleExit handleHook\n");
@@ -347,7 +412,6 @@ static void handleExit()
 
     freeTimers(NULL);  // can we create timer in (Exit) ???
     freeVars();
-    freeCachedData();
     freeCfg();
     freeRegexps();
     dbusFinish();
@@ -378,7 +442,7 @@ static int handle_alarm(char* key)
     type_key *k = findItem(key , &i, &params);
 
     if (k && i == FLAG_EXACT) {
-        handleCmdByKeyEx(k, NULL,0);
+        handleCmdByKeyEx(PEER_ANY,k, NULL, 0);
 
         if (flushConf == 1) {
             logger(L_DBG, "[EX]: Flush old configuration");
@@ -393,18 +457,49 @@ static int handle_alarm(char* key)
     return EXIT_OK;
 }
 
-static int handle_key_press(char* key,int isRepeatable)
+static int handle_key_press(int peer, char* key, int isRepeatable)
 {
     DEBUG2("[EX]: handle_key_press() >%s<", (key ? key :  "NULL"));
     if (!key) {
         return EXIT_OK;
     }
+    
+    if (peer != 0 && strncmp("CoverSize(",key,10) == 0) {
+        int v = 0;
+        int i = 10;
+        while (isdigit(*(key+i))) {
+            v = v*10 + (*(key+i) - '0');
+            i++;
+        }
+        DEBUG2("[EX]: CoverSize: %d >%d<", peer, v);
+        customizePeer(peer, -1, -1, v);
+    }
+    if (peer != 0 && strncmp("SizeX(",key,6) == 0) {
+        int v = 0;
+        int i = 6;
+        while (isdigit(*(key+i))) {
+            v = v*10 + (*(key+i) - '0');
+            i++;
+        }
+        DEBUG2("[EX]: SizeX: %d >%d<", peer, v);
+        customizePeer(peer, v, -1, -1);
+    }
+    if (peer != 0 && strncmp("SizeY(",key,6) == 0) {
+        int v = 0;
+        int i = 6;
+        while (isdigit(*(key+i))) {
+            v = v*10 + (*(key+i) - '0');
+            i++;
+        }
+        DEBUG2("[EX]: SizeY: %d >%d<", peer, v);
+        customizePeer(peer, -1, v, -1);
+    }
 
     // If modifier was set already it needs to verify which sequence we go
     if (modifierString[0] != '\0') {
         strcat(modifierString," "); // Add one space first
         strcat(modifierString, key);
-        DEBUG2("Modifier+sequence is: >%s<", modifierString);
+        DEBUG2("[EX]: Modifier+sequence is: >%s<", modifierString);
     }
 
     // Execute appropriate command
@@ -414,35 +509,35 @@ static int handle_key_press(char* key,int isRepeatable)
 
     if (k && i == FLAG_EXACT) {
         if (isRepeatable && (!repeatNow()) && getAutoRepeat()) {
-            logger(L_DBG, "Set auto repeat flag");
+            logger(L_DBG, "[EX]: Set auto repeat flag");
             setRepeatNow(k);
         }
-        handleCmdByKey(k, NULL);
+        handleCmdByKey(peer, k, NULL);
 
         // Clean modifier string
         modifierString[0] = '\0';
 
         if (flushConf == 1) {
-            logger(L_DBG, "Flush old configuration");
+            logger(L_DBG, "[EX]: Flush old configuration");
             flushConf = 0;
 
             flushOldConf();
         }
 
     } else if (k && i == FLAG_MULTIKEY) {
-        logger(L_DBG, "Got part of multi keys sequence. Nothing to do.");
+        logger(L_DBG, "[EX]: Got part of multi keys sequence. Nothing to do.");
         if (modifierString[0] == '\0') {
-            logger(L_DBG, "Start of multi key sequence");
+            logger(L_DBG, "[EX]: Start of multi key sequence");
             strcpy(modifierString, key);
         }
 
-        if (needAtMainMenuReturn()) {
-            sendToMainMenu();
+        if (needAtMainMenuReturn(peer)) {
+            sendToMainMenu(peer);
         }
     } else if (k && i == FLAG_PARAMETR) {
-        logger(L_DBG, "Got parametrized command");
+        logger(L_DBG, "[EX]: Got parametrized command");
 
-        handleCmdByKey(k, &params);
+        handleCmdByKey(peer, k, &params);
 
         // Clean modifier string
         modifierString[0] = '\0';
@@ -450,105 +545,112 @@ static int handle_key_press(char* key,int isRepeatable)
     } else {
         // User had pressed some key ...
         // Send ToMainMenu sequence of CKPD's to show main menu again
-        if (needAtMainMenuReturn()) {
-            sendToMainMenu();
+        if (needAtMainMenuReturn(peer)) {
+            sendToMainMenu(peer);
         }
 
-        logger(L_DBG, "No approprite key definition was found");
+        logger(L_DBG, "[EX]: No approprite key definition was found");
         if (modifierString[0] != '\0') {
             // Clean modifier string
             modifierString[0] = '\0';
-            logger(L_DBG, "Clean modifier string");
+            logger(L_DBG, "[EX]: Clean modifier string");
         }
     }
 
     return EXIT_OK;
 }
 
-static int handle_command(char* cmd)
+static int handle_at_command(int peer, char* cmd)
 {
-    logger(L_DBG,"[EX]: handle_command");
+    // Handle different keypresses
+    // Motorola: +CKEV: "1",1
+    // SE	   : +CKEV: 1,1
+    // Sagem: in rare ) cases it could be (between > and <) >,1:< ?
 
+    int isSE = MODEL_SE;
+
+    char *key = cmd+7;
+
+    // Sagem test
+    // key+=3;
+
+    if (*key == '"') {	// Seems this is Motorola
+        key++;
+        //logger(L_DBG,"[EX]: +CKEV is in Motorola format");
+        isSE = MODEL_MOTOROLA;
+    }
+
+    if (*key == ',') {	// Seems this is Sagem
+        logger(L_DBG,"[EX]: +CKEV is in rare Sagem format (Empty code)");
+        isSE = MODEL_SAGEM;
+    }
+
+    int i = 1;
+
+    if (isSE == MODEL_SE) { 	// SonyEricsson & default
+        while(key[i] != ',') {
+            i++;
+        }
+        key[i] ='\0';
+
+        i = i+1;
+    } else if (isSE == MODEL_MOTOROLA) {	// Motorola
+        while(key[i] != '"') {
+            i++;
+        }
+        key[i] ='\0';
+
+        i = i+2;
+    } else if (isSE == MODEL_SAGEM) {	// rare Sagem case
+        *key ='\0';
+    } else {
+        logger(L_ERR,"[EX]: Can't recognize +CKEV event !!!");
+        return EXIT_NOK;
+    }
+
+    if (key[i] == '1') {
+        //logger(L_DBG,"[EX]: Button down event");
+        
+        dMessage* dm = allocDMessage();
+        dm->type     = DM_EVENT;
+        dm->subtype  = ID_EVENT_FRONTEND;
+        dm->value    = (void*) strdup(key);
+        dm->size     = strlen(key);
+
+        sendToDispatcher(dm);
+	
+        return handle_key_press(peer,key,1);
+    
+    } else if (key[i] == '0') {
+        
+        //logger(L_DBG,"[EX]: Button up event");
+        // In general we skip this
+        cleanAutoRepeatFlag();
+        return EXIT_OK;
+    
+    } 
+    
+    //else 
+    DEBUG2("[EX]: something wrong: key should be pressed or released: cmd=>%s< char=>%c<", cmd, key[i]);
+    return EXIT_NOK;
+}
+
+static int handle_command(int peer, char* cmd)
+{
+    //logger(L_DBG,"[EX]: handle_command");
     if (!cmd) {
         return EXIT_OK;
     }
-
-    char *key = NULL;
-
-    if (memcmp(cmd, DEF_CLDEBUG, 12) == 0) {	    // Got debug event from Java client
-
-        return EXIT_OK;
-
-    } else if (memcmp(cmd, DEF_MSG, 4) == 0) {	    // Got event from Java client
-
-        key = cmd+4;
+    
+    if (memcmp(cmd, DEF_MSG, 4) == 0) {	    // Got event from Java client
 
         logger(L_DBG,"[EX]: Got event from client");
-        return handle_key_press(key,0);
+        return handle_key_press(peer, cmd+4, 0);
 
     } else if (memcmp(cmd, DEF_CKEV, 6) == 0) {
 
-        // Handle different keypresses
-        // Motorola: +CKEV: "1",1
-        // SE	   : +CKEV: 1,1
-        // Sagem: in rare ) cases it could be (between > and <) >,1:< ?
-
-        int isSE = MODEL_SE;
-
-        key = cmd+7;
-
-        // Sagem test
-        // key+=3;
-
-        if (*key == '"') {	// Seems this is Motorola
-            key++;
-            //logger(L_DBG,"[EX]: +CKEV is in Motorola format");
-            isSE = MODEL_MOTOROLA;
-        }
-
-        if (*key == ',') {	// Seems this is Sagem
-            logger(L_DBG,"[EX]: +CKEV is in rare Sagem format (Empty code)");
-            isSE = MODEL_SAGEM;
-        }
-
-        int i = 1;
-
-        if (isSE == MODEL_SE) { 	// SonyEricsson & default
-            while(key[i] != ',') {
-                i++;
-            }
-            key[i] ='\0';
-
-            i = i+1;
-        } else if (isSE == MODEL_MOTOROLA) {	// Motorola
-            while(key[i] != '"') {
-                i++;
-            }
-            key[i] ='\0';
-
-            i = i+2;
-        } else if (isSE == MODEL_SAGEM) {	// rare Sagem case
-            *key ='\0';
-        } else {
-            logger(L_ERR,"[EX]: Can't recognize +CKEV event !!!");
-            return EXIT_NOK;
-        }
-
-        if (key[i] == '1') {
-            //logger(L_DBG,"[EX]: Button down event");
-            sendToFrontEnd(key);
-	    
-            return handle_key_press(key,1);
-        } else if (key[i] == '0') {
-            //logger(L_DBG,"[EX]: Button up event");
-            // In general we skip this
-            cleanAutoRepeatFlag();
-
-            return EXIT_OK;
-        } else {
-            DEBUG2("[EX]: something wrong: key should be pressed or released: cmd=>%s< char=>%c<",cmd, key[i]);
-            return EXIT_NOK;
-        }
+        return handle_at_command(peer, cmd);
+    
     } else if (memcmp(cmd, DEF_CLCC,6) == 0) {
 
         //logger(L_INF,"[EX]: Caller ID received. Skip it.");
@@ -556,7 +658,7 @@ static int handle_command(char* cmd)
 
     } else if (strstr(cmd, DEF_RING) != NULL) {	    // Incoming call, this event sent periodically until used answered a call
         
-	logger(L_INF,"[EX]: Incoming call notification. Skip it."); // Handled inside peer read function
+        logger(L_INF,"[EX]: Incoming call notification. Skip it."); // Handled inside peer read function
         return EXIT_OK;
 	
     } else if (isLircCommand(cmd)) {	    // LIRC tricks
@@ -568,8 +670,8 @@ static int handle_command(char* cmd)
     } else if (isIViewerCommand(cmd)) {	    // CommandFusion iViewer protocol support
         return EXIT_OK;
     } else { // SERVER_STDIN ? Just direct input, try it
+        
         //printf("DEBUG: Is it SERVER_STDIN ? %s\n", cmd);
-
         char *p = cmd;
         while (*p != '\0') {
             if (*p == '\n') {
@@ -579,7 +681,7 @@ static int handle_command(char* cmd)
             p++;
         }
 
-        return handle_key_press(cmd,1);
+        return handle_key_press(peer,cmd,1);
     }
 
     DEBUG2("[EX]: Unhandled cmd %s", cmd);
@@ -589,17 +691,18 @@ static int handle_command(char* cmd)
 //
 // Should be used in AT mode only
 //
-void sendToMainMenu()
+void sendToMainMenu(int peer)
 {
     logger(L_INF,"[EX]: sendToMainMenu");
     char *tmm = NULL;
     if ((tmm = getToMainMenu()) != NULL) {
 
-        dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
-        dm->type  = DM_CKPD;
-        dm->value = (void*) tmm;
-        dm->size  = strlen((char*) dm->value);
-        dm->file  = NULL;
+        dMessage* dm = allocDMessage();
+        dm->peer     = peer;
+        dm->type     = DM_CKPD;
+        dm->subtype  = ID_SET_MAX;
+        dm->value    = (void*) tmm;
+        dm->size     = strlen((char*) dm->value);
 
         sendToDispatcher(dm);
     }
@@ -733,7 +836,7 @@ static int isBemusedCommand(char *cmdIn)
 
             DEBUG2("[EX]: isBemusedCommand: one commmand >%s<", oneCmd);
 
-            handle_key_press(oneCmd,1);
+            handle_key_press(PEER_ANY,oneCmd,1);
         } else {
             break;
         }
@@ -778,7 +881,7 @@ static int isLircCommand(char *cmdIn)
         free(cmd);
         return 0;
     }
-    handle_key_press(token,1);
+    handle_key_press(PEER_ANY,token,1);
 
     free(cmd);
     return 1;
@@ -831,7 +934,7 @@ static int isInpLircCommand(char *cmdIn)
 
     if (isLirc) {
         DEBUG2("[EX]: isLircCommand: one commmand >%s<", data);
-        handle_key_press(data,1);
+        handle_key_press(PEER_ANY,data,1);
     }
 
     free(cmd);
@@ -907,7 +1010,7 @@ static int isIViewerCommand(char *cmdIn)
     free(cmdTmp);
 
     DEBUG2("[EX]: isIViewerCommand: one commmand >%s<", cmd);
-    handle_key_press(cmd,1);
+    handle_key_press(PEER_ANY,cmd,1);
 
     logger(L_INF, "[EX]: isIViewerCommand: EXIT");
     free(cmd);
