@@ -2,11 +2,11 @@
 // anyRemote
 // a wi-fi or bluetooth remote for your PC.
 //
-// Copyright (C) 2012-2014 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2016 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #include <ctype.h>
@@ -52,8 +52,6 @@ extern boolean_t stillRun;
 typedef struct _SocketConnection_ {
     int           fileDescriptor;
     int           serverFileDescriptor;
-    int           ivInCommand;
-    int           ivHeartbeat;
     char*         uSocket;
 } _SocketConnection;
 
@@ -67,7 +65,8 @@ int socketFD(ConnectInfo* conn)
     if (!cn) {
         return -1;
     }
-    return (conn->state == PEER_WAIT_ACCEPT ? cn->serverFileDescriptor : cn->fileDescriptor);
+    return (conn->state == PEER_WAIT_ACCEPT || 
+            conn->state == PEER_WAIT_LISTEN ? cn->serverFileDescriptor : cn->fileDescriptor);
 }
 
 static int socketOpenInternal(ConnectInfo* conn)
@@ -81,7 +80,7 @@ static int socketOpenInternal(ConnectInfo* conn)
     int sz;
 
     if (conn->connectionData && ((_SocketConnection*) conn->connectionData)->serverFileDescriptor > 0) {
-        logger(L_ERR, "TCP/local socket was already opened");
+        logger(L_ERR, "[DS]: socketOpenInternal: TCP/local socket was already opened");
         return 1;
     }
     
@@ -95,8 +94,6 @@ static int socketOpenInternal(ConnectInfo* conn)
     
     cn->serverFileDescriptor = -1;
     cn->fileDescriptor       = -1;
-    cn->ivInCommand          = 0;
-    cn->ivHeartbeat          = 0;
     cn->uSocket              = NULL;
     
     const char *path = (conn->portStr ? conn->portStr->str : NULL);
@@ -108,16 +105,16 @@ static int socketOpenInternal(ConnectInfo* conn)
         addFamily = AF_UNIX;
         proto     = 0;
     } else {
-        logger(L_ERR, "incorrect input");
+        logger(L_ERR, "socketOpenInternal: incorrect input");
         return -1;
     }
 
     if ((cn->serverFileDescriptor = socket(addFamily, SOCK_STREAM|SOCK_CLOEXEC, proto)) < 0) {
         
-        logger(L_ERR, "opening TCP socket");
+        logger(L_ERR, "[DS]: socketOpenInternal: opening TCP socket");
         errnoDebug("opening TCP socket ",errno);  // testing debug
         
-        printf("ERROR: opening TCP socket\n");
+        printf("ERROR: socketOpenInternal: opening TCP socket\n");
         cn->serverFileDescriptor = -1;
         return -1;
     }
@@ -145,12 +142,12 @@ static int socketOpenInternal(ConnectInfo* conn)
         sz = sizeof(un_addr);
 
     } else {
-        logger(L_ERR, "incorrect input 2");
+        logger(L_ERR, "socketOpenInternal: incorrect input 2");
         return -1;
     }
 
     if (bind(cn->serverFileDescriptor, (struct sockaddr *) socketaddr, sz) < 0) {
-        logger(L_ERR, "on bind()");
+        logger(L_ERR, "[DS]: socketOpenInternal: on bind()");
         printf("ERROR: on bind socket to the address %s (%d)\n", strerror(errno), errno);
         return -1;
     }
@@ -164,17 +161,19 @@ static int socketOpenInternal(ConnectInfo* conn)
 int socketOpen(ConnectInfo* conn)
 {
     if (conn->mode == SERVER_TCP) {
-        DEBUG2("[DS]: TCP mode. Use port %d", conn->port);
+        DEBUG2("[DS]: socketOpen: TCP mode. Use port %d", conn->port);
     } else if (conn->mode == SERVER_UX) {
-        DEBUG2("[DS]: Local socket mode. Use file %s", conn->portStr->str);
+        DEBUG2("[DS]: socketOpen: Local socket mode. Use file %s", conn->portStr->str);
     } else {
-        DEBUG2("[DS]: improper mode. %d", conn->mode);
+        DEBUG2("[DS]: socketOpen: improper mode. %d", conn->mode);
         return EXIT_NOK;
     }
 
     if (socketOpenInternal(conn) < 0) {
         return EXIT_NOK;
-    }
+    } 
+    
+    conn->state = PEER_WAIT_LISTEN;
     return EXIT_OK;
 }
 
@@ -184,19 +183,19 @@ void socketClose(ConnectInfo* conn, int final)
     if (!cn) return;
     
     if (final) {
-        logger(L_INF, "closeSocket");
+        logger(L_INF, "[DS]: closeSocket");
     }
     
     if (cn->fileDescriptor >= 0) {
         if (final) {
-            logger(L_INF, "closeSocket close socket");
+            logger(L_INF, "[DS]: closeSocket close socket");
         }
         close(cn->fileDescriptor);
         cn->fileDescriptor = -1;
     }
     if (cn->serverFileDescriptor >= 0) {
         if (final) {
-            logger(L_INF, "closeSocket close server socket");
+            logger(L_INF, "[DS]: closeSocket close server socket");
         }
         close(cn->serverFileDescriptor);
         cn->serverFileDescriptor = -1;
@@ -213,12 +212,26 @@ void socketClose(ConnectInfo* conn, int final)
     conn->state = PEER_DISCONNECTED;
 }
 
-//
-// Setup socket (listen)
-//
-int socketSetup(ConnectInfo* conn)
+void socketReset(ConnectInfo* conn)
 {
-    logger(L_INF, "[DS]: Server mode: setup peer");
+    _SocketConnection* cn = (_SocketConnection*) conn->connectionData;
+    if (cn) {
+        if (cn->fileDescriptor >= 0) {
+            close(cn->fileDescriptor);
+            cn->fileDescriptor = -1;
+        }
+        conn->state = PEER_WAIT_ACCEPT;
+    } else {
+        conn->state = PEER_DISCONNECTED;  // should not happens
+    }
+}
+
+//
+// Setup listen
+//
+int socketListen(ConnectInfo* conn)
+{
+    logger(L_INF, "[DS]: socketListen");
     
     _SocketConnection* cn = (_SocketConnection*) conn->connectionData;
     if (!cn) {
@@ -230,7 +243,7 @@ int socketSetup(ConnectInfo* conn)
     }
     return (ret < 0 ? -1 : 1);
 }
-        
+     
 //
 // Wait for incoming connection
 //
@@ -243,7 +256,7 @@ int socketAccept(ConnectInfo* conn)
     int cnt, sz;
     int type = conn->mode;
     
-    logger(L_INF, "[DS]: socketAccept");
+    INFO2("[DS]: socketAccept %d", conn->id);
     cnt = 0;
 
     if (type == SERVER_TCP) {
@@ -263,7 +276,7 @@ int socketAccept(ConnectInfo* conn)
 
     while (stillRun) {
 
-        INFO2("[DS]: socketAccept: accept on %d", cn->serverFileDescriptor);
+        INFO2("[DS]: socketAccept: accept on %d", conn->id);
  
         cn->fileDescriptor = accept(cn->serverFileDescriptor, (struct sockaddr *) socketaddr, (socklen_t *)&sz);
 
@@ -347,16 +360,20 @@ int socketAccept(ConnectInfo* conn)
     
         INFO2("[DS]: socketAccept: accepted from %s", buf);
         conn->state = PEER_CONNECTED;
+        
+        // force to detect H/W and cover size. need to do that before (Connect) or syncPeer() handling
+        getClientSize(conn->id, cn->fileDescriptor);  
 
         break;
     }
+    
     logger(L_INF, "[DS]: socketAccept exit");
     return 1;
 }
 
 int socketWrite(ConnectInfo* connInfo, const dMessage* msg)
 {
-    //logger(L_DBG, "socketWrite");
+    logger(L_DBG, "[DS]: socketWrite");
     if (!msg) {
         return EXIT_OK;
     }
@@ -372,15 +389,23 @@ int socketWrite(ConnectInfo* connInfo, const dMessage* msg)
     }
     
     _SocketConnection* cn = (_SocketConnection*) connInfo->connectionData;
-    if (!cn) return EXIT_NOK;
+    if (!cn) {
+        return EXIT_NOK;
+    }
 
     // send command
     if (cn->fileDescriptor >= 0) {
+    
+        if (msg->type == DM_SET) {
+            if (isDataOld(connInfo, msg->subtype, command, count)) {
+                INFO2("[DS]: Skip to send the same data to TCP peer");
+                return EXIT_OK;
+            }
+        }
 
         memset(tmp, 0, MAXMAXLEN);
-        strcat(tmp, "socketWrite ");
+        strcat(tmp, "[DS]: socketWrite ");
 
-        cn->ivInCommand = 1;
         int logSz = (count > 256 ? 255 : count);
 
         // it is possible to get binary data here
@@ -388,17 +413,17 @@ int socketWrite(ConnectInfo* connInfo, const dMessage* msg)
         tmp[logSz] = '\0';
         logger(L_DBG, tmp);
 
-        sprintf(tmp, "socketWrite %d bytes", count);
+        sprintf(tmp, "[DS]: socketWrite %d bytes", count);
         logger(L_INF, tmp);
 
         int n = write(cn->fileDescriptor,command,count);
         if (n < 0) {
-            logger(L_ERR, "error writing to socket");
+            logger(L_ERR, "[DS]: error writing to socket");
             return EXIT_NOK;
         }
         return EXIT_OK;
     } else {
-        logger(L_ERR, "error writing to socket: already closed");
+        logger(L_ERR, "[DS]: error writing to socket: already closed");
     }
     return EXIT_NOK;
 
