@@ -2,11 +2,11 @@
 // anyRemote
 // a wi-fi or bluetooth remote for your PC.
 //
-// Copyright (C) 2011-2014 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2016 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #include <ctype.h>
@@ -61,20 +61,11 @@
 
 extern char tmp[MAXMAXLEN];
 extern int  gotExitSignal;
+extern boolean_t stillRun;
 
-static int       visits       = 0;          // counts client connections
-static boolean_t runWeb      = BOOL_YES;
-static boolean_t answerReady = BOOL_NO;
+static int       visits       = 0;          // counts client requests
+static boolean_t answerReady  = BOOL_NO;
    
-typedef struct _WebConnection_ {
-    int fileDescriptor;
-    int refreshPage;  // refrech time in seconds, -1 meand no refresh    
-    int secure;
-    long cookie;
-    char* confDir;
-    string_t* serverIP;
-    SingleList *clientSockets;
-} _WebConnection;
 
 typedef struct _WebClientConnection_ {
     int serverPort;    
@@ -93,6 +84,15 @@ typedef struct _WebClientConnection_ {
 #define IVIEW_HEAD3 "</commandPort>\n    </controlsystem>\n    <size>\n      <portrait width=\"320\" height=\"480\" />\n      <landscape width=\"480\" height=\"320\" />\n    </size>\n    <imagefolder>\n    </imagefolder>\n    <cache images=\"0\" />\n    <debug loaderrors=\"0\" />\n  </properties>"
 #define IVIEW_TAIL  "</gui>"
 
+/*int haveClients(_WebConnection* cn)
+{
+    mutexLock(M_WEB);
+    int num = listSingleLength(cn->clientSockets);
+    mutexUnlock(M_WEB);
+    
+    DEBUG2("[WS]: clients #%d", num);
+    return (num > 0 ? 1 : 0);
+}*/    
 
 static const char* i2string(int data,char* buf)
 {
@@ -126,6 +126,7 @@ static struct {
 } _mimeTypes[] = {
     {".html", "text/html"},
     {".htm",  "text/html"},
+    {".xml",  "text/xml" },
     {".jpg",  "image/jpeg"},
     {".jpeg", "image/jpeg"},
     {".gif",  "image/gif"},
@@ -550,6 +551,7 @@ void sendForm(_WebClientConnection* cc, string_t* content, boolean_t cookie)
     }
 
     sendData(cc, header->str); 
+    //printf("%s",header->str); 
     
     stringFree(header, BOOL_YES);
 
@@ -558,6 +560,7 @@ void sendForm(_WebClientConnection* cc, string_t* content, boolean_t cookie)
     }
 
     sendData(cc, "\r\n");
+    //printf("\r\n"); 
 
     sendData(cc, content->str);
     //printf("%s",content->str);
@@ -1113,13 +1116,15 @@ static void renderForm(int tvisits, _WebClientConnection* cc)
     }
 }
 
-
+// separate thread for each request
 pointer_t serveRequest(pointer_t data)
 {
     char buf[4096];
     char *method = NULL;
     char *path   = NULL;
     struct stat statbuf;
+    
+    signal(SIGPIPE, SIG_IGN);  // why id did not inherit this ?
 
     _WebClientConnection* cc = (_WebClientConnection*) data;
 
@@ -1167,6 +1172,7 @@ pointer_t serveRequest(pointer_t data)
 
     char * line       = NULL;
     char * screenDef  = NULL;
+    char * screenDef2 = NULL;
     while ((line = strtok(NULL, "\r"))) {
         if (strstr(line,"Cookie:")) {
             cookieLine = line;
@@ -1178,6 +1184,9 @@ pointer_t serveRequest(pointer_t data)
         if (cc->runMode == SERVER_CMXML && strstr(line,"x-CiscoIPPhoneDisplay:")) {
             screenDef = line;
         }
+        if (cc->runMode == SERVER_CMXML && strstr(line,"x-CiscoIPPhoneModelName:")) {
+            screenDef2 = line;
+        }
     }
 
     //parse path for GET request" GET /path HTTP/1.1
@@ -1199,6 +1208,9 @@ pointer_t serveRequest(pointer_t data)
     if (screenDef) {
         parseScreenDef(screenDef);
     }
+    if (screenDef2) {
+        parseCiscoModel(screenDef2);
+    }
 
     char* p = NULL;
 
@@ -1216,6 +1228,13 @@ pointer_t serveRequest(pointer_t data)
         } else if (strcmp(path,"/")  == 0 || strcmp(path,"\n") == 0) {
 
             renderForm(tvisits, cc);
+            //wait = BOOL_YES;
+
+        } else if (strncmp(path,"/?name=SEP", 10) == 0) {
+        
+            // URL: GET /?name=SEP0014F2C23169&locale=United_States_English HTTP/1.0
+            renderForm(tvisits, cc);
+            //wait = BOOL_YES;
 
         } else if (strcmp(path,"/favicon.ico") == 0) {
 
@@ -1434,7 +1453,7 @@ pointer_t serveRequest(pointer_t data)
             stringFree(fpath, BOOL_YES);
         }
     }
-
+    
     if (wait) {
         mutexLock(M_WEB);
         answerReady = BOOL_NO;
@@ -1462,7 +1481,7 @@ pointer_t serveRequest(pointer_t data)
     //INFO2("[WS]: (%d) clientSockets after remove #%d", tvisits, listSingleLength(cc->conn->clientSockets));
     
     free(data);
-    
+
     mutexUnlock(M_WEB);
 
     INFO2("[WS]: (%d) Request processed", tvisits);
@@ -1471,8 +1490,20 @@ pointer_t serveRequest(pointer_t data)
     return (void*)0;
 }
 
+int webFD(ConnectInfo* conn)
+{
+    _WebConnection* cn = (_WebConnection*) conn->connectionData;
+    if (!cn) {
+        return -1;
+    }
+    return (conn->state == PEER_WAIT_ACCEPT || 
+            conn->state == PEER_WAIT_LISTEN ? cn->serverFileDescriptor : -1);
+}
+
 static void freeConn(ConnectInfo* conn)
 {
+    INFO2("[WS]: freeConn");
+    
     _WebConnection* cn = (_WebConnection*) conn->connectionData;
     
     mutexLock(M_WEB);
@@ -1491,9 +1522,9 @@ static void freeConn(ConnectInfo* conn)
     
     mutexUnlock(M_WEB);
 
-    if (cn->fileDescriptor != -1) {
-        close(cn->fileDescriptor);
-        cn->fileDescriptor = -1;
+    if (cn->serverFileDescriptor != -1) {
+        close(cn->serverFileDescriptor);
+        cn->serverFileDescriptor = -1;
     }
 
     if (cn->serverIP) {
@@ -1510,19 +1541,26 @@ static void freeConn(ConnectInfo* conn)
     conn->connectionData = NULL;
 }
 
-static pointer_t webServerMain(pointer_t data)
+static int openWebInternal(ConnectInfo* conn)
 {
-    ConnectInfo* conn = (ConnectInfo*) data;
-    _WebConnection* cn = (_WebConnection*) conn->connectionData;
-    
-    cn->cookie  = 0;
-    cn->secure  = NO_COOKIE;
-    cn->confDir = getCfgDir();
-    cn->clientSockets = NULL;
-    cn->serverIP      = NULL;
-    
+    INFO2("[WS]: openWebInternal");
+    mutexNew(M_WEB); 
     queueNew(Q_WEB);
     
+    if (conn->connectionData) {
+        freeConn(conn);
+    }
+    
+    conn->connectionData = (_WebConnection*) malloc(sizeof(_WebConnection));
+    _WebConnection* cn = (_WebConnection*) conn->connectionData;
+    
+    cn->serverFileDescriptor = -1;
+    cn->cookie         = 0;
+    cn->secure         = NO_COOKIE;
+    cn->confDir        = dupVarValue(VAR_CFGDIR);
+    cn->clientSockets  = NULL;
+    cn->serverIP       = NULL;
+
     if (conn->mode == SERVER_WEB) {
         initHtmlGenerator();
     }
@@ -1537,9 +1575,8 @@ static pointer_t webServerMain(pointer_t data)
     } else {
         cn->refreshPage = -1;
     }
-
     INFO2("[WS]: $(RefreshPage) = %d", cn->refreshPage);
-
+    
     char* v3 = dupVarValue("IpAddr");
     if (v3 != NULL) {
         INFO2("[WS]: $(IpAddr) = %s", v3);
@@ -1559,70 +1596,91 @@ static pointer_t webServerMain(pointer_t data)
     struct sockaddr_in sin;
     memset((void *) &sin, 0, sizeof(sin));
 
-    cn->fileDescriptor = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
+    cn->serverFileDescriptor = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 
-    if (cn->fileDescriptor < 0) {
-        logger(L_ERR, "[WS]: webServerMain: Failed to create a socket");
-
-        freeConn(conn);
-    
-        threadExit(T_WEB);
-        return (void*)0;
+    if (cn->serverFileDescriptor < 0) {
+        logger(L_ERR, "[WS]: Failed to create a socket for web server");
+        return -1;
     }
 
     int optval = 1;
-    setsockopt(cn->fileDescriptor, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
+    setsockopt(cn->serverFileDescriptor, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
 
     sin.sin_family = AF_INET;
     sin.sin_addr.s_addr = INADDR_ANY;
     sin.sin_port = htons(conn->port);
-    int ret = bind(cn->fileDescriptor, (struct sockaddr *) &sin, sizeof(sin));
+    int ret = bind(cn->serverFileDescriptor, (struct sockaddr *) &sin, sizeof(sin));
     if (ret < 0) {
         logger(L_ERR, "[WS]: webServerMain: Failed to bind a socket");
         printf("ERROR: on binding %d->%s\n", errno, strerror(errno));
-
-        gotExitSignal = 1;
-        
-        freeConn(conn);
-
-        threadExit(T_WEB);
-        return (void*)0;
+        return -1;
     }
+    
 
-    listen(cn->fileDescriptor, 100);
-    INFO2("[WS]: webServerMain: HTTP server listening on port %d", conn->port);
+    return 0;
+}
 
-    while (runWeb) {
+int openWeb(ConnectInfo* conn)
+{
+    DEBUG2("[DS]: Web Server mode. Use port %d", conn->port);
+    if (openWebInternal(conn) < 0) {
+        conn->state = PEER_DISCONNECTED;
+        return EXIT_NOK;
+    }
+     
+    conn->state = PEER_WAIT_LISTEN;
+    return EXIT_OK;
+}
 
-        logger(L_INF, "[WS]: webServerMain: HTTP server ready to accept connection");
+int listenWeb(ConnectInfo* conn)
+{
+    logger(L_INF, "[DS]: Server mode: listen from web peer ");
+    
+    _WebConnection* cn = (_WebConnection*) conn->connectionData;
+    if (!cn) {
+        return -1;
+    }
+    
+    int ret = listen(cn->serverFileDescriptor, 100);  // allow many request at once
+    if (ret >= 0) {
+        conn->state = PEER_WAIT_ACCEPT;
+    }
+    return (ret < 0 ? -1 : 1);
+}
 
-        int s = accept(cn->fileDescriptor, NULL, NULL);
+//
+// Wait for incoming connection
+//
+int acceptWeb(ConnectInfo* conn)
+{
+    logger(L_INF, "[WS]: acceptWeb");
+    
+    _WebConnection* cn = (_WebConnection*) conn->connectionData;
+    if (!cn) {
+        return -1;
+    }
+    
+    while (stillRun) {
+
+        logger(L_INF, "[WS]: HTTP server ready to accept connection");
+
+        int s = accept(cn->serverFileDescriptor, NULL, NULL);
         if (s < 0) {
             errnoDebug("[WS]: accept() ",errno);  // testing debug
             if (errno == EAGAIN) {
                 continue;
             } else {
-                break;
+                return -1;
             }
         }
 
         char buf[INET6_ADDRSTRLEN];
         peerName(s,buf,INET6_ADDRSTRLEN);
  
-        INFO2("[WS]: webServerMain: HTTP server accepts connection from %s", buf);
+        INFO2("[WS]: acceptWeb: HTTP server accepts connection from %s", buf);
         if (isAllowed(buf)) {
-            INFO2("[WS]: webServerMain: connection accepted");
-            
-            /*if (conn->state == PEER_WAIT_ACCEPT) {
-                wMessage* wm = (wMessage*) malloc(sizeof(wMessage));
-                wm->button = -1;
-                wm->string = stringNew("(Connect)");
-
-                sendToWebServer(wm);
-                
-                conn->state = PEER_CONNECTED;
-            }*/
-            
+            INFO2("[WS]: acceptWeb: connection accepted");
+             
             _WebClientConnection* ptr = (_WebClientConnection*) malloc(sizeof(_WebClientConnection));
             ptr->serverPort     = conn->port;
             ptr->connDescriptor = s;
@@ -1635,55 +1693,17 @@ static pointer_t webServerMain(pointer_t data)
 
             int rq = threadNew(T_MAX, serveRequest, (void *) ptr, DETACHED);
             if (rq != RC_OK) {
-                logger(L_ERR, "[WS]: webServerMain: Can not run processing thread");
+                logger(L_ERR, "[WS]: acceptWeb: Can not run processing thread");
             }
         } else {
-            INFO2("[WS]: webServerMain: host %s is not in the list of accepted host, skip connection", buf);
+            INFO2("[WS]: acceptWeb: host %s is not in the list of accepted host, skip connection", buf);
             close(s);
         }
-    }
-
-    logger(L_ERR, "[WS]: webServerMain: exit thread");
-    freeConn(conn);
-
-    threadExit(T_WEB);
-    return (void*)0;
-}
-
-static int openWebInternal(ConnectInfo* conn)
-{
-    if (threadExists(T_WEB) == RC_OK) {
-        INFO2("[WS]: openWebInternal, skip initialization");
-        return 0;
-    }
-
-    mutexNew(M_WEB);
-    
-    if (conn->connectionData) {
-        freeConn(conn);
+        break;
     }
     
-    conn->connectionData = (_WebConnection*) malloc(sizeof(_WebConnection));
-    _WebConnection* cn = (_WebConnection*) conn->connectionData;
-    cn->fileDescriptor = -1;
-
-
-    threadNew(T_WEB,webServerMain,conn,JOINABLE);
-
-    return 0;
-}
-
-int openWeb(ConnectInfo* conn)
-{
-    DEBUG2("[DS]: Web Server mode. Use port %d", conn->port);
-    if (openWebInternal(conn) < 0) {
-        conn->state = PEER_DISCONNECTED;
-        return EXIT_NOK;
-    }
-    
-     
-    conn->state = PEER_CONNECTED; //PEER_WAIT_ACCEPT;
-    return EXIT_OK;
+    logger(L_INF, "[WS]: acceptWeb OK");
+    return 1;
 }
 
 int checkWebPort(char* buf, int capacity)
@@ -1729,21 +1749,20 @@ int checkWebPort(char* buf, int capacity)
 }
 
 
-void closeWebPort(ConnectInfo* conn, int final)
+void webClose(ConnectInfo* conn, int final)
 {
-    INFO2("[WS]: closeWebPort %d", final);
-    if (final) {
+    INFO2("[WS]: webClose %d", final);
+    
+    //if (final) {  -- if app started then it will keep HTTP/XML connection opened
+    
         _WebConnection* cn = (_WebConnection*) conn->connectionData;
+        INFO2("[WS]: webClose serverIP %s", cn->serverIP ? cn->serverIP->str : "NULL");
 
         // close all sockets
-        if (cn->fileDescriptor != -1) {
-            close(cn->fileDescriptor);
-            cn->fileDescriptor = -1;
-        }
 
         mutexLock(M_WEB);
 
-        //INFO2("[WS]: clientSockets before cleanup #%d", listSingleLength(cn->clientSockets));
+        INFO2("[WS]: clientSockets before cleanup #%d", listSingleLength(cn->clientSockets));
 
         SingleList* list = cn->clientSockets;
         while (list) {
@@ -1760,9 +1779,16 @@ void closeWebPort(ConnectInfo* conn, int final)
 
         mutexUnlock(M_WEB);
 
-        runWeb = BOOL_NO;
+        if (cn->serverFileDescriptor >= 0) {
+            if (final) {
+                logger(L_INF, "closeSocket close web server socket");
+            }
+            close(cn->serverFileDescriptor);
+            cn->serverFileDescriptor = -1;
+        }
 
         if (cn->serverIP) {
+            INFO2("[WS]: serverIP %s", cn->serverIP->str);
             stringFree(cn->serverIP, BOOL_YES);
         }
 
@@ -1770,14 +1796,22 @@ void closeWebPort(ConnectInfo* conn, int final)
             free(cn->confDir);
             cn->confDir = NULL;
         }
-
         mutexRemove(M_WEB);
         queueRemove(Q_WEB, freeWMessage);
-    }
-
+        
+        free(conn->connectionData);
+        conn->connectionData = NULL;
+    //}
+    
+    conn->state = PEER_DISCONNECTED;
     return;
 }
 
+void webReset(ConnectInfo* conn)
+{
+    webClose(conn, 0);
+}
+
 int writeWebConnStr(const char* value, int mode)
 {
     DEBUG2("[WS]: writeWebConnStr %d", (value ? (int) strlen(value) : -1));
@@ -1821,7 +1855,7 @@ int writeWebConnStr(const char* value, int mode)
 
         wm = (wMessage*) malloc(sizeof(wMessage));
         wm->button = -1;
-        wm->string = stringNew("SizeX(");
+        wm->string = stringNew("Msg:SizeX(");
     
         const char *buf = i2string(w,b);
         stringAppend(wm->string,buf);
@@ -1831,7 +1865,7 @@ int writeWebConnStr(const char* value, int mode)
         wm2 = (wMessage*) malloc(sizeof(wMessage));
         wm2->button = -1;
     
-        wm2->string = stringNew("SizeY(");
+        wm2->string = stringNew("Msg:SizeY(");
     
         buf = i2string(h,b);
         stringAppend(wm2->string,buf);
@@ -1845,7 +1879,7 @@ int writeWebConnStr(const char* value, int mode)
 
         wm = (wMessage*) malloc(sizeof(wMessage));
         wm->button = -1;
-        wm->string = stringNew("IconPadding(");
+        wm->string = stringNew("Msg:IconPadding(");
         stringAppend(wm->string,buf);
         stringAppend(wm->string,",)");
 
@@ -1857,22 +1891,44 @@ int writeWebConnStr(const char* value, int mode)
 
     } else if (strncmp(token,"Get(is_exists",13) == 0) {
 
-        char* sz = strtok(NULL,",");
-        char* icon = strtok(NULL,",");
-        if (sz && icon) {
+        char* what = strtok(NULL,",");
+        char* sz   = strtok(NULL,",");
+          
+        if (sz && what) {
+            
+            int isCover = strcmp("cover",what);
+
             while (isspace(*sz)) {
                 ++sz;
             }
-            while (isspace(*icon)) {
-                ++icon;
+            while (isspace(*what)) {
+                ++what;
+            }
+            
+            char* icon = NULL;
+            if (isCover != 0) {
+                icon = strtok(NULL,",");
             }
 
             wm = (wMessage*) malloc(sizeof(wMessage));
             wm->button = -1;
-            wm->string = stringNew("IconExists(");
-            stringAppend(wm->string,sz);
-            stringAppend(wm->string,",");
-            stringAppend(wm->string,icon);
+            
+            if (isCover  == 0) {
+             
+                wm->string = stringNew("Msg:CoverExists(,");
+                stringAppend(wm->string,sz);
+           
+            } else if (icon) {
+            
+                while (isspace(*icon)) {
+                    ++icon;
+                }
+                
+                wm->string = stringNew("Msg:IconExists(");
+                stringAppend(wm->string,sz);
+                stringAppend(wm->string,",");
+                stringAppend(wm->string,icon);
+            }
             stringAppend(wm->string,")");
         }
 
@@ -1880,7 +1936,7 @@ int writeWebConnStr(const char* value, int mode)
 
         wm = (wMessage*) malloc(sizeof(wMessage));
         wm->button = -1;
-        wm->string = stringNew("CoverSize(");
+        wm->string = stringNew("Msg:CoverSize(");
         char b[32];
 
         stringAppend(wm->string,
@@ -1892,7 +1948,7 @@ int writeWebConnStr(const char* value, int mode)
 
         wm = (wMessage*) malloc(sizeof(wMessage));
         wm->button = -1;
-        wm->string = stringNew("Version(,web_interface)");
+        wm->string = stringNew("Msg:Version(,web_interface)");
 
     } else if (strncmp(token,"Get(cursor",10) == 0) {
         // ignore
@@ -1924,8 +1980,9 @@ int writeWebConnStr(const char* value, int mode)
 
 int writeWebConn(const dMessage* dm, int mode)
 {
-    INFO2("[WS]: writeWebConn (%d)", dm->size);
-    if (dm->type == DM_SET) {
+    INFO2("[WS]: writeWebConn %d (%d)", dm->type, dm->size);
+    if (dm->type == DM_SET ||
+        dm->type == DM_GET) {
 
         if (strncmp(dm->value,"Get(",4) == 0 ||   // Set(...) command updates state inside peer.c : writePeers()
             strncmp(dm->value,"End();",6) == 0) {  
@@ -1947,10 +2004,20 @@ int writeWebConn(const dMessage* dm, int mode)
 //
 int writeWeb(ConnectInfo* peer, const dMessage* dm)
 {
+    INFO2("[WS]: writeWeb (%d)", dm->size);
     // suppose peer->mode == SERVER_WEB || peer->mode == SERVER_CMXML
     if (dm->type == DM_SENDB || dm->type == DM_SENDS) {
+        INFO2("[WS]: writeWeb SKIP");
         return EXIT_NOK;  // skip that
     }
+    
+    if (dm->type == DM_SET) {
+        if (isDataOld(peer, dm->subtype, dm->value, dm->size)) {
+            INFO2("[DS]: Skip to send the same data to WEB/CMXML peer");
+            return EXIT_OK;
+        }
+    }
+
     return writeWebConn(dm, peer->mode);
 }
 
