@@ -2,11 +2,11 @@
 // anyRemote
 // a wi-fi or bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2014 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2016 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #include <arpa/inet.h>
@@ -371,7 +371,7 @@ static int pclose_r(int fd, pid_t pid)
     return status;
 }
 
-// this version seems works
+// seems this version works
 
 char* executeCommandPipe(const char* exec, size_t* sz)
 {
@@ -402,11 +402,11 @@ char* executeCommandPipe(const char* exec, size_t* sz)
         size_t num = read(fd, temp, RSIZE);
 
         if (num == -1 && errno == EAGAIN)  {
- 	    //DEBUG2("[  ]: executeCommandPipe EAGAIN");
-	    usleep(1000);
+ 	        //DEBUG2("[  ]: executeCommandPipe EAGAIN");
+	        usleep(1000);
         } else if (num > 0) {
 	
-	    //DEBUG2("[  ]: executeCommandPipe read %d", num);
+	        //DEBUG2("[  ]: executeCommandPipe read %d", num);
             data = realloc(data, haveRead+num+1);
             if (!data) {
                 DEBUG2("[  ]: executeCommandPipe error on realloc %zd", haveRead+num);
@@ -416,20 +416,20 @@ char* executeCommandPipe(const char* exec, size_t* sz)
             haveRead += num;
             *(data + haveRead) = '\0';
         } else {
-	    break;
-	}
+	        break;
+	    }
 
         //DEBUG2("[  ]: executeCommandPipe realloc %zd", haveRead);
-	
-	if (maxWait > 0) {
-	    time_t now = time(NULL); 
-	    double elapsed = difftime(now, start);
-	    if (elapsed > maxWait) {
-	        WARNING2("[  ]: executeCommandPipe wait time exceeded");
-		kill(pid, SIGTERM);
-		break;
+
+	    if (maxWait > 0) {
+	        time_t now = time(NULL); 
+	        double elapsed = difftime(now, start);
+	        if (elapsed > maxWait) {
+	            WARNING2("[  ]: executeCommandPipe wait time exceeded");
+		        kill(pid, SIGTERM);
+		        break;
             }
-	}
+	    }
     }
 
     pclose_r(fd, pid);
@@ -567,6 +567,8 @@ string_t* executeDirListCommand(int type, const char* directory)
 	
     } else if (S_ISDIR(buf.st_mode)) {
     
+        DEBUG2("[EX]: executeDirListCommand: directory");
+        
         dirTmp = directory;  // small trick
     
         struct dirent **namelist = NULL;
@@ -584,13 +586,13 @@ string_t* executeDirListCommand(int type, const char* directory)
 
 		        if (strcmp(namelist[i]->d_name,".") != 0) {
 		            if (comma == 1) {
-			        stringAppend(folders,",");
+			            stringAppend(folders,",");
 		            } else {
 		                comma = 1;
 		            }
 
  		            if (type == ID_SET_ILIST) {
-			        stringAppend(folders,"folder:");
+			            stringAppend(folders,"folder:");
 		            }
 		            stringAppend(folders,namelist[i]->d_name);
 		            stringAppend(folders,"/");
@@ -599,103 +601,104 @@ string_t* executeDirListCommand(int type, const char* directory)
             }
             free(namelist);
         }
+        //DEBUG2("[EX]: executeDirListCommand: %s", folders->str);
 	
 	    n = scandir(directory, &namelist, filterFiles, alphasort);
         if (n >= 0) {
 	
-	    int i = 0;
- 	    for( ; i<n; i++) {
-        	//printf("%s\n", namelist[i]->d_name);
-		
-		if (strcmp(namelist[i]->d_name,".") != 0) {
-		    if (comma == 1) {
-			stringAppend(folders,",");
-		    } else {
-		        comma = 1;
-		    }
-		    
- 		    if (type == ID_SET_ILIST) {
-			stringAppend(folders,"file:");
-		    }
-		    stringAppend(folders,namelist[i]->d_name);
-		}
-        	free(namelist[i]);
+	        int i = 0;
+ 	        for( ; i<n; i++) {
+        	    //printf("%s\n", namelist[i]->d_name);
+
+		        if (strcmp(namelist[i]->d_name,".") != 0) {
+		            if (comma == 1) {
+			            stringAppend(folders,",");
+		            } else {
+		                comma = 1;
+		            }
+
+ 		            if (type == ID_SET_ILIST) {
+			            stringAppend(folders,"file:");
+		            }
+		            stringAppend(folders,namelist[i]->d_name);
+		        }
+                free(namelist[i]);
             }
             free(namelist);
         }
-	
-	return folders;
-
-	/* gets items unsorted
-	DIR *dirp = opendir(directory);
-	
-	if (dirp) {
-	
-	    struct dirent *dp;
-	    
-            int fComma = 0;
-            int dComma = 0;
-	    string_t *files   = stringNew("");
-	    string_t *folders = stringNew("");
-	   
-	    string_t *file    = stringNew("");
-	    
-	    while ((dp=readdir(dirp))) {
-		
-		if (strcmp(dp->d_name,".") == 0) continue;
-		
-		//DEBUG2("[EX]: executeDirListCommand process %s", dp->d_name);
-		
-		stringTruncate(file,0);
-		stringAppend(file,directory);
-		stringAppend(file,"/");
-		stringAppend(file,dp->d_name);
-		
-		int stat = lstat(file->str, &buf);
-		if (stat < 0) continue;
-		
-		if (S_ISDIR(buf.st_mode)) {
-
- 		    if (dComma == 1) {
-			stringAppend(folders,",");
-		    } else {
-		        dComma = 1;
-		    }
-
- 		    if (type == ID_SET_ILIST) {
-			stringAppend(folders,"folder:");
-		    }
-		    stringAppend(folders,dp->d_name);
-		    stringAppend(folders,"/");
-		    
-	        } else //if (S_ISREG(buf.st_mode) || S_ISLNK(buf.st_mode))
-		      {
-		    
-		    if (fComma == 1) {
-			stringAppend(files,",");
-		    } else {
-		        fComma = 1;
-		    }
-
-		    if (type == ID_SET_ILIST) {
-			stringAppend(files,"file:");
-		    }
-		    stringAppend(files,dp->d_name);
-
-		}
-            }
-	    
-	    stringFree(file, BOOL_YES);
-            closedir(dirp);
-	    
-	    if (folders->len > 0) {
-	        stringAppend(folders,",");
-	    }
-	    stringAppend(folders,files->str);  // dirs, then files
-	    stringFree(files,   BOOL_YES);
-	    
-	    //DEBUG2("[EX]: executeDirListCommand DONE");
+        //DEBUG2("[EX]: executeDirListCommand: %s", folders->str);
 	    return folders;
+
+	    /* gets items unsorted
+	    DIR *dirp = opendir(directory);
+
+	    if (dirp) {
+
+	        struct dirent *dp;
+
+                int fComma = 0;
+                int dComma = 0;
+	        string_t *files   = stringNew("");
+	        string_t *folders = stringNew("");
+
+	        string_t *file    = stringNew("");
+
+	        while ((dp=readdir(dirp))) {
+
+		    if (strcmp(dp->d_name,".") == 0) continue;
+
+		    //DEBUG2("[EX]: executeDirListCommand process %s", dp->d_name);
+
+		    stringTruncate(file,0);
+		    stringAppend(file,directory);
+		    stringAppend(file,"/");
+		    stringAppend(file,dp->d_name);
+
+		    int stat = lstat(file->str, &buf);
+		    if (stat < 0) continue;
+
+		    if (S_ISDIR(buf.st_mode)) {
+
+ 		        if (dComma == 1) {
+			    stringAppend(folders,",");
+		        } else {
+		            dComma = 1;
+		        }
+
+ 		        if (type == ID_SET_ILIST) {
+			    stringAppend(folders,"folder:");
+		        }
+		        stringAppend(folders,dp->d_name);
+		        stringAppend(folders,"/");
+
+	            } else //if (S_ISREG(buf.st_mode) || S_ISLNK(buf.st_mode))
+		          {
+
+		        if (fComma == 1) {
+			    stringAppend(files,",");
+		        } else {
+		            fComma = 1;
+		        }
+
+		        if (type == ID_SET_ILIST) {
+			    stringAppend(files,"file:");
+		        }
+		        stringAppend(files,dp->d_name);
+
+		    }
+                }
+
+	        stringFree(file, BOOL_YES);
+                closedir(dirp);
+
+	        if (folders->len > 0) {
+	            stringAppend(folders,",");
+	        }
+	        stringAppend(folders,files->str);  // dirs, then files
+	        stringFree(files,   BOOL_YES);
+
+	        //DEBUG2("[EX]: executeDirListCommand DONE");
+	        return folders;
         }*/
     }   
     return NULL; 
@@ -746,10 +749,12 @@ string_t* getLocalIP()
     int  i;
     for(i = 0; i < nifaces; i++) {
         DEBUG2("[WS]: interface #%d is %s", i, ifreqs[i].ifr_name);
-        if (strcmp("lo",ifreqs[i].ifr_name) == 0) {
+        static char* lo = "lo";
+        if (strcmp(lo,ifreqs[i].ifr_name) == 0) {
             continue;  // skip loopback
         }
-        if (strcmp("127.0.0.1",inet_ntoa(((struct sockaddr_in *)&ifreqs[i].ifr_addr)->sin_addr)) == 0) {
+        static char* loopback = "127.0.0.1";
+        if (strcmp(loopback, inet_ntoa(((struct sockaddr_in *)&ifreqs[i].ifr_addr)->sin_addr)) == 0) {
             continue;  // skip loopback
         }
         DEBUG2("[WS]: use local IP %s", inet_ntoa(((struct sockaddr_in *)&ifreqs[i].ifr_addr)->sin_addr));
