@@ -2,11 +2,11 @@
 // anyRemote
 // a wi-fi or bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2014 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2016 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
 #include <errno.h>
@@ -25,7 +25,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-//#include <unistd.h>
 
 #include "common.h"
 #include "lib_wrapper.h"
@@ -44,11 +43,6 @@ extern CONF         conf;
 // Some globals
 
 char logfile [MAXLEN];
-static char tofile[MAXLEN];
-static char cmdfile[MAXLEN];
-
-char * lastValues    [ID_SET_MAX] = {NULL};
-int    lastValuesSize[ID_SET_MAX] = {-1};
 
 // used by Flush() command to temporary store old configuration
 static mode       *flushModes   = NULL;
@@ -73,26 +67,41 @@ iconv_t convertorFrom = (iconv_t) -1;
 static void initFile(char* what, const char* name)
 {
     // store data in first arg
-
-    char *prefix = getenv("AR_TMPDIR");
-
+   
+    int useuser = 0;
+    
+    char *prefix = dupVarValue("TmpDir");
+    char *h = getenv("HOME");  
+    
     if (prefix) {
         strcat(what, prefix);
+        if (h) {
+            if (strncmp(prefix, h, strlen(h)) != 0) {
+                useuser = 1;
+            }
+        } else {
+            useuser = 1;
+        }
     } else {
-        char *h = getenv("HOME");
         if (h) {
             strcat(what, h);
             strcat(what, "/.anyRemote");
         } else {			// could it ever happen ?
             strcat(what, "/tmp");
+            useuser = 1;
         }
     }
     strcat(what, name);
-    char *u = getenv("USER");
-    if (prefix && u) {
-        strcat(what, ".");
-        strcat(what, u);
+    
+    if (useuser == 1) {
+        char *u = getenv("USER");
+        if (prefix && u) {
+            strcat(what, ".");
+            strcat(what, u);
+        }
     }
+    
+    free(prefix);
     return;
 }
 
@@ -211,88 +220,6 @@ void printTime()
     logger(L_CFG, timestr);
 }
 
-void setCmdFiles()
-{
-    initFile(tofile,  TOFILE);
-    initFile(cmdfile, CMDFILE);
-}
-
-char* getResFile()
-{
-    return tofile;
-}
-
-char* getCmdFile()
-{
-    return cmdfile;
-}
-
-//////////////////////////////////////////////////////////////////////////////////
-//
-// To minimize data transfer cache results for some commands and send data only if data were changed
-//
-//////////////////////////////////////////////////////////////////////////////////
-
-static boolean_t isDataCacheable(int what) 
-{
-    if (what == ID_SET_BG      ||
-        what == ID_SET_CAPTION ||
-        what == ID_SET_FG      ||
-        what == ID_SET_FONT    ||
-        what == ID_SET_ICONS   ||
-        what == ID_SET_LAYOUT  ||
-        what == ID_SET_STATUS  ||
-        what == ID_SET_TITLE   ||
-        what == ID_SET_VOLUME  ||
-        what == ID_SET_COVER) {
-        return BOOL_YES;
-    } else {
-        return BOOL_NO;
-    }
-}
-
-int isDataNew(int what, const char *data, int size)
-{
-    if (!isDataCacheable(what)) {
-        return RC_OK;				// consider as new value
-    }
-    
-    DEBUG2("isDataNew(): %d -> %s", what, (data?data:"NULL"));
-
-    int isNew = RC_NOK;
-    if (lastValues[what] == NULL) {
-        isNew = RC_OK;		     	// consider as new value
-    } else if (lastValues[what] != NULL &&
-               (lastValuesSize[what] != size ||
-                memcmp(lastValues[what],data,lastValuesSize[what]) != 0)) {
-
-        free(lastValues[what]);
-        lastValues[what]     = NULL;
-        lastValuesSize[what] = -1;
-
-        isNew = RC_OK;		     	// consider as new value
-    }
-
-    if (isNew == RC_OK) {
-        lastValues[what] = (char*) calloc(size + 1, 1);
-        memcpy(lastValues[what],data,size);
-        lastValuesSize[what] = size;
-    }
-    return isNew;
-}
-
-void freeCachedData()
-{
-    //DEBUG2("freeCachedData()");
-    int i = 0;
-    for (; i<ID_SET_MAX; i++) {
-        if (lastValues[i]) {
-            free(lastValues[i]);
-            lastValues[i] = NULL;
-        }
-    }
-}
-
 //////////////////////////////////////////////////////////////////////////////////
 //
 // Functions related to encoding conversion
@@ -314,11 +241,11 @@ void closeConvertor(void)
 {
     if (convertorTo != (iconv_t) -1) {
         iconv_close(convertorTo);
-	convertorTo = (iconv_t) -1;
+	    convertorTo = (iconv_t) -1;
     }
     if (convertorFrom != (iconv_t) -1) {
         iconv_close(convertorFrom);
-	convertorFrom = (iconv_t) -1;
+	    convertorFrom = (iconv_t) -1;
     }
 }
 
@@ -361,11 +288,6 @@ void encodingHook()
 
 extern int  flushConf;
 
-//void printCfgDir()
-//{
-//	printf("%s\n", getCfgDir()?getCfgDir():"directory with configuration files can not be determined");
-//}
-
 int flushData()
 {
     //logger(L_DBG,"flushData");
@@ -380,8 +302,6 @@ int flushData()
 
     flushTimers = getTimers();
     forgetTimers();
-    
-    freeCachedData();
 
     // Now we ready to load new cfg.
     return EXIT_OK;
@@ -403,14 +323,15 @@ void flushOldConf()
 void printHelp()
 {
     printf("Usage: anyremote [-f conf.file] [-s peer[,peer ...]] [-log] [-a] \n");
+    printf("                 [-cfgdir directory] [-tmpdir directory]\n");
     printf("                 [-fe port] [-u username] [-name SDP service name] [-password]\n");
     printf("   or: anyremote -h|-v\n\n");
     printf(" -h|--help  print this help\n");
     printf(" -v|--version  print version of anyRemote\n");
     printf(" -f /path/to/file  specify configuration file\n");
     printf(" -s peer[,peer ...], where peer possible values are\n");
-    printf("    bluetooth:<channel> (Server mode - bluetooth connection)\n");
     printf("    tcp:<port>          (Server mode - TCP/IP connection)\n");
+    printf("    bluetooth:<channel 1-32> (Server mode - bluetooth connection)\n");
     printf("    web:<port>          (Server mode - Web interface)\n");
     printf("    cmxml:<port>        (Server mode - XML interface for Cisco IP phones)\n");
     printf("    local:/dev/ircommX  (Server mode - IR connection)\n");
@@ -422,10 +343,12 @@ void printHelp()
     printf("    ilirc:<AF_LOCAL socket file> (use with inputlircd)\n");
     printf("    stdin\n");
     printf("    avahi - register SDP service using Avahi\n");
-    printf("  Default peer value is bluetooth:19,tcp:5197,web:5080\n");
+    printf("  Default peer value is bluetooth,tcp:5197,web:5080,avahi\n");
     printf("  It is possible to specify several peers for Server mode configuration files only\n");
     printf("  It is possible to specify only single peer of web: or cmxml: type\n");
     printf(" -log print verbose logging information to $HOME/.anyRemote/anyremote.log\n");
+    printf(" -cfgdir override default location of directory with configuration files\n");
+    printf(" -tmpdir override default location of directory to store temporary files\n");
     printf(" -a	reconnect automatically in case of connection failure, used only in AT-mode\n");
     printf(" -fe <port> work as backend for GUI frontend. Use specified port to connect to frontend.\n");
     printf(" -name <SDP service name> if bluetooth or TCP/IP connection is used, allows one to specify SDP service name\n");
